
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2010 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//
//  Abstract/Purpose:
//    Header file for Data Engine 5.0 or greater. This file defines the
//    manageable data events which are monitored by the engine
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCDATAEVENT_H_INCLUDED__
#define __DCDATAEVENT_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// Shared include files
//--------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmstatus.h>
#include <dcdataobj.h>


#ifdef __cplusplus
extern "C" {
#endif


#pragma pack(1)


//---------------------------------------------------------------------
//  Structures and definitions
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// DEFINITIONS
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Limits
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/** Size in bytes of data event information used to define the event */
//---------------------------------------------------------------------
#define MAX_DATA_EVENT_DEF_SIZE			(16)

//---------------------------------------------------------------------
/** Minimum size in bytes of data event */
//---------------------------------------------------------------------
#define MIN_DATA_EVENT_SIZE				(sizeof(DataEventHeader))

//---------------------------------------------------------------------
/**
 * Maximum size in bytes of data event (default) including header,
 *	some bytes of data and a data object
 */
//---------------------------------------------------------------------
#define MAX_DATA_EVENT_SIZE				(MIN_DATA_EVENT_SIZE + MAX_DATA_EVENT_DEF_SIZE + MAX_DATA_OBJECT_SIZE)

//---------------------------------------------------------------------
/** Maximum size in bytes of the data event body */
//---------------------------------------------------------------------
#define MAX_DATA_EVENT_BODY_SIZE		(MAX_DATA_EVENT_SIZE - MIN_DATA_EVENT_SIZE)


//---------------------------------------------------------------------
/**
 * Data Event types assigned by Data Manager team
 * Definition of manageable event types (DataEventHeader.evtType)
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 0 is an invalid event type.
 *  No event can contain this type. This is ignored. */
//---------------------------------------------------------------------
#define DATA_EVENT_TYPE_INVALID			(0)


//---------------------------------------------------------------------
/** 1 - 1023 are predefined, intercepted and processed event types by
 *  Data Manager that cause a specific action to be taken */
//---------------------------------------------------------------------

/** Minimum in predefined range */
#define DATA_EVENT_TYPE_BASE_LO			(1)

/** Maximum in predefined range */
#define DATA_EVENT_TYPE_BASE_HI			(1023)


//---------------------------------------------------------------------
/** 1024 - 2047 are reserved event types for Disney (HIP) */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range */
#define DATA_EVENT_TYPE_HIP_LO			(1024)

/** Maximum in Disney (HIP) range */
#define DATA_EVENT_TYPE_HIP_HI			(2047)


//---------------------------------------------------------------------
/** 2048 - 3071 are reserved event types for Storage Services */
//---------------------------------------------------------------------

/** Minimum in Storage Services range */
#define DATA_EVENT_TYPE_STORAGE_SVCS_LO	(2048)

/** Maximum in Storage Services range */
#define DATA_EVENT_TYPE_STORAGE_SVCS_HI	(3071)


//---------------------------------------------------------------------
/** 3072 - 3327 are reserved event types for License Management */
//---------------------------------------------------------------------

/** Minimum in License Management range */
#define DATA_EVENT_TYPE_LICENSE_MGMT_LO	(3072)

/** Maximum in License Management range */
#define DATA_EVENT_TYPE_LICENSE_MGMT_HI	(3327)


//---------------------------------------------------------------------
/** 3328 - 65533 are event types available for assignment by Data Manager team.
 *  Should be considered reserved by all other teams.
 *  Contact Data Manager team to be assigned an event type range. */
//---------------------------------------------------------------------

/** Minimum in assignable range */
#define DATA_EVENT_TYPE_NEXT_AVAILABLE_LO	(3328)

/** Maximum in assignable range */
#define DATA_EVENT_TYPE_NEXT_AVAILABLE_HI	(65533)


//---------------------------------------------------------------------
/** 65534 is reserved by Data Manager for further redefinition of the DataEventHeader */
//---------------------------------------------------------------------
#define DATA_EVENT_TYPE_RESERVED1		(65534)


//---------------------------------------------------------------------
/** 65535 is reserved by Data Manager.  Mostly liked to be used by test tools. */
//---------------------------------------------------------------------
#define DATA_EVENT_TYPE_RESERVED2		(65535)



//---------------------------------------------------------------------
/** Event Flags for Event Header (EventObjHeader.evtFlags) */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Used to reset event flags */
//---------------------------------------------------------------------
#define DATA_EVENT_FLAG_NONE			(0)

//---------------------------------------------------------------------
/**
 * On receiving an event with this property the Data Manager then
 * examines the evtType field to determine further action. If the
 * evtType is a predefined base event then the Data Manager immediately
 * processes it. Base event types are explained in the HLD.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_FLAG_BASE			BIT(0)

//---------------------------------------------------------------------
/**
 * On receiving this event the Data Manager interprets the event to be
 * of the type an intra Data Populator event. In this case the event
 * type is not examined by the Data Manager. It simply passes the
 * event on to all Data Populator modules currently registered using
 * the SM_DRT_POP_SET_DATA_EVENT message.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_FLAG_INTRA_POP		BIT(1)

//---------------------------------------------------------------------
/**
 * On receiving this event the Data Manager interprets the event to
 * be of the type broadcast which means that it will pass the event
 * on to all registered Data Consumer modules (as a normal event)
 * and all registered Data Populator (as an intra Populator event).
 * However it will not examine the evtType field itself.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_FLAG_POP_TO_CONSUMER	BIT(2)

//---------------------------------------------------------------------
/**
 * On receiving this event the Data Manager interprets the event to
 * be of the type broadcast which means that it will first parse the
 * event as in SM_EVENT_FLAG_BASE. Then it moves on to treating it
 * like SM_EVENT_FLAG_INTRA_POP. Then moves on to treating like
 * SM_EVENT_FLAG_POP_TO_CONSUMER.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_FLAG_BROADCAST		(DATA_EVENT_FLAG_BASE | \
										DATA_EVENT_FLAG_INTRA_POP | \
										DATA_EVENT_FLAG_POP_TO_CONSUMER)

//---------------------------------------------------------------------
/** Reserved for future use */
//---------------------------------------------------------------------
#define DATA_EVENT_FLAG_FLAG_RESERVED	(BIT(3) | \
										BIT(4) | \
										BIT(5) | \
										BIT(6) | \
										BIT(7))



//---------------------------------------------------------------------
/** Base predefined event types (DATA_EVENT_FLAG_BASE) must be set in flags */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Create a new object tree or data object with this OID if not
 * exists under single parent Used by older data Populators.
 * @note Should be considered OBSOLETE
 */
//---------------------------------------------------------------------
#define DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID	(DATA_EVENT_TYPE_BASE_LO + 0)

//---------------------------------------------------------------------
/**
 * Delete a new object tree or data object with this OID if exists.
 * Used by older data Populators.
 * @note Should be considered OBSOLETE
 */
//---------------------------------------------------------------------
#define DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID	(DATA_EVENT_TYPE_BASE_LO + 1)

//---------------------------------------------------------------------
/**
 * This event type is generated from Data Populator to Data Manager
 * in order to refresh a list of Data Objects. This list is specified
 * in the form of an Object Identifier list. The evtSize of the Event
 * Data object is a minimum of DATA_EVENT_DL_MIN_SIZE. The event data
 * body is made up of objList to be refreshed. The Data Manager will
 * call SM_DRT_POP_REFRESH_OBJ on each Data Object in the list
 * indicated  for the refresh.
 * @see DataObjRefreshMultiple, DataObjRefreshSingle
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_OBJ_DIRTY_LIST		(DATA_EVENT_TYPE_BASE_LO + 2)

//---------------------------------------------------------------------
/**
 * This event type is generated by the Data Manager for consumption
 * by Data Consumer modules interested in receiving Data Object status
 * change events. When the Data Manager detects a change in the Data
 * Object status it generates this event to all registered consumers.
 * This event contains the previous object status and redundancy
 * status if the object type is redundancy object. It also contains
 * the copy of the Data Object before the status changed. This
 * allows a consumer to use this copy of the object to compare
 * with the latest copy of the object which can be obtained using
 * SM_DRT_COM_GET_OBJ_BY_OID. The evtSize of the Event Data object
 * is a minimum of DATA_EVENT_OSC_MIN_SIZE. The event data body is
 * made up of objStatusChange to be refreshed.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_OBJ_STATUS_CHG			(DATA_EVENT_TYPE_BASE_LO + 3)

//---------------------------------------------------------------------
/**
 * This event is sent by DCSGEN to all queued event listeners when
 * the consumer is attached (made connection) to the data manager
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_CONSUMER_ATTACHED	(DATA_EVENT_TYPE_BASE_LO + 4)

//---------------------------------------------------------------------
/**
 * This event is sent by DCSGEN to all queued event listeners when
 * the consumer is detached (lost connection) to the data manager
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_CONSUMER_DETACHED	(DATA_EVENT_TYPE_BASE_LO + 5)

//---------------------------------------------------------------------
/**
 * This event is sent by data manager to all listening data consumers
 * to indicate that the list of data objects (OID) included in the
 * event have been created. This event type is generated from
 * Data Populator (older populators) to Data Manager in order
 * to insert a new Data Object. The evtSize of the Event Data
 * object is DATA_EVENT_SCOBO_SIZE. The event data body is made
 * up of objID. When this Data Event is received by the Data Manager
 * it will perform an existence check on parentID and objID. Then
 * obtain and create the object employing the Data Populator using
 * SM_DRT_COM_GET_OBJ_BY_OID.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_OBJ_CREATE			(DATA_EVENT_TYPE_BASE_LO + 6)

//---------------------------------------------------------------------
/**
 * This event type is generated from Data Populator to Data Manager
 * in order to delete an existing Data Object. The evtSize of the
 * Event Data object is DATA_EVENT_SDOBO_SIZE. The event data body
 * is made up of delObjID. When received by the Data Manager it
 * verifies the delObjID for existence and then deletes the object.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_OBJ_DESTROY			(DATA_EVENT_TYPE_BASE_LO + 7)

//---------------------------------------------------------------------
/**
 * This event is sent by data manager to all listening data consumers
 * to indicate that the list of data objects (OID) included in the
 * event have branches created
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_OBJ_BRANCH_CREATE	(DATA_EVENT_TYPE_BASE_LO + 8)

//---------------------------------------------------------------------
/**
 * This event is sent by data manager to all listening data consumers
 * to indicate that the list of data objects (OID) included in the
 * event have branches destroyed
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_OBJ_BRANCH_DESTROY	(DATA_EVENT_TYPE_BASE_LO + 9)

//---------------------------------------------------------------------
/**
 * This event is sent by DCSTOR to all queued event listeners and data
 * populators when the data manager startup completes
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_MANAGER_ATTACHED	(DATA_EVENT_TYPE_BASE_LO + 10)

//---------------------------------------------------------------------
/**
 * This event is sent by DCSTOR to all queued event listeners and data
 * populators when the data manager shutdown starts
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DATA_MANAGER_DETACHED	(DATA_EVENT_TYPE_BASE_LO + 11)

//---------------------------------------------------------------------
/**
 * The sub tree status of the specified data object (OID) whose status
 * is data populator managed has changed and the suggested object
 * status for this data object is included in the event to the data
 * populator responsible for object
 */
//---------------------------------------------------------------------
#define DATA_EVENT_SUGGEST_POPMNGD_OBJ_STATUS	(DATA_EVENT_TYPE_BASE_LO + 12)

//---------------------------------------------------------------------
/**
 * Power event that is sent by Data Manager to indicate that the system
 * is suspending operation.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_POWER_SUSPEND			(DATA_EVENT_TYPE_BASE_LO + 13)

//---------------------------------------------------------------------
/**
 * Power event that is sent by Data Manager to indicate that the system
 * is resuming operation.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_POWER_RESUME				(DATA_EVENT_TYPE_BASE_LO + 14)

//---------------------------------------------------------------------
/**
 * Device change event that is sent by Data Manager to indicate that
 * a device has been added to the system.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DEVICE_ADD				(DATA_EVENT_TYPE_BASE_LO + 15)

//---------------------------------------------------------------------
/**
 * Device change event that is sent by Data Manager to indicate that
 * a device has been removed from the system.
 */
//---------------------------------------------------------------------
#define DATA_EVENT_DEVICE_REMOVE			(DATA_EVENT_TYPE_BASE_LO + 16)



//---------------------------------------------------------------------
/** Limits on size of event buffers for event types defined above */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID @note: OBSOLETE */
//---------------------------------------------------------------------
#define DATA_EVENT_SCOBO_SIZE			(MIN_DATA_EVENT_SIZE + (2 * sizeof(ObjID)))

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID @note: OBSOLETE */
//---------------------------------------------------------------------
#define DATA_EVENT_SDOBO_SIZE			(MIN_DATA_EVENT_SIZE + sizeof(ObjID))

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_DATA_OBJ_DIRTY_LIST minium size (1 OID) */
//---------------------------------------------------------------------
#define DATA_EVENT_DL_MIN_SIZE			(MIN_DATA_EVENT_SIZE + sizeof(ObjList))

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_OBJ_STATUS_CHG */
//---------------------------------------------------------------------
#define DATA_EVENT_OSC_MIN_SIZE			(MIN_DATA_EVENT_SIZE + 4 + sizeof(DataObjHeader))

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_DATA_MANAGER_ATTACHED */
//---------------------------------------------------------------------
#define DATA_EVENT_DMA_SIZE				(MIN_DATA_EVENT_SIZE)

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_DATA_MANAGER_DETACHED */
//---------------------------------------------------------------------
#define DATA_EVENT_DMD_SIZE				(MIN_DATA_EVENT_SIZE)

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_DATA_CONSUMER_ATTACHED */
//---------------------------------------------------------------------
#define DATA_EVENT_DCA_SIZE				(MIN_DATA_EVENT_SIZE)

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_DATA_CONSUMER_DETACHED */
//---------------------------------------------------------------------
#define DATA_EVENT_DCD_SIZE				(MIN_DATA_EVENT_SIZE)

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_DATA_OBJ_CREATE, DATA_EVENT_DATA_OBJ_DESTROY */
//---------------------------------------------------------------------
#define DATA_EVENT_DOCD_MIN_SIZE		(MIN_DATA_EVENT_SIZE + sizeof(ObjList))

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_DATA_OBJ_BRANCH_CREATE, DATA_EVENT_DATA_OBJ_BRANCH_DESTROY */
//---------------------------------------------------------------------
#define DATA_EVENT_DOBCD_MIN_SIZE		(MIN_DATA_EVENT_SIZE + sizeof(ObjList) + sizeof(ObjID))

//---------------------------------------------------------------------
/** Size for Event Type: DATA_EVENT_SUGGEST_POPMNGD_OBJ_STATUS */
//---------------------------------------------------------------------
#define DATA_EVENT_SPOS_SIZE			(MIN_DATA_EVENT_SIZE + 4 + sizeof(ObjID))


//--------------------------------------------------------------------
// STRUCTURES
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Data Structure that apply to all data events
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Structure the is present at the begining of a Data Event.
 * All data events contain a header followed by a body. A Data Event
 * is used to indicate a message or event in the Data Manager and
 * its Data Populators and Data Consumers
 */
//---------------------------------------------------------------------
typedef struct _DataEventHeader
{
	/** Total size of the event object, inclusive of evtSize. */
	u32 evtSize;
	/** This determines the type of event and union to be selected. */
	u16 evtType;
	/** This specifies the attributes of the event object to data manager. */
	u8 evtFlags;
	/** Reserved for future use. Should be set to 0 by data populator. Checked by Data Manager. */
	u8 reservedAlign[1];
	/** The time this event was generated got from c-lib function time() */
	s64 evtTimeStamp;
} DataEventHeader;


//--------------------------------------------------------------------
/** Base Event structures */
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Data Event structure for single parent object created by OID
 * for DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID
 * @note OBSOLETE
 */
//---------------------------------------------------------------------
typedef struct _DESingleParentCreateObj
{
	/** Parent under which to create child. */
	ObjID parentID;
	/** Child object id to be created (will cause a SM_DRT_COM_GET_OBJ_BY_OID to Data Populator). */
	ObjID objID;
} DESingleParentCreateObj;


//---------------------------------------------------------------------
/**
 * Data Event structure for object status change notification
 * for DATA_EVENT_OBJ_STATUS_CHG
 */
//---------------------------------------------------------------------
typedef struct _DEObjStatusChange
{
	/** Previous object status of enclosed data object. */
	u8 prevObjStatus;
	/** Previous redundanacy status. If enclosed data object type == DATA_OBJ_TYPE_REDUNDANCY. Invalid otherwise. */
	u8 prevRedStatus;
	/** Alignment for next data element */
	u8 reservedAlign[2];
	/** Enclosed copy of the data object contents at the time of this event (entire object if cached or non-cached) */
	DataObjHeader objHeader;
} DEObjStatusChange;


//---------------------------------------------------------------------
/**
 * Data Event structure used for data populator managed object
 * suggested status change notification
 * for DATA_EVENT_SUGGEST_POPMNGD_OBJ_STATUS
 */
//---------------------------------------------------------------------
typedef struct _DEObjSuggestPopManagedStatus
{
	/** Suggested object status of enclosed data object. */
	u8 suggestObjStatus;
	/** Suggested redundanacy status. If enclosed data object type == DATA_OBJ_TYPE_REDUNDANCY. Invalid otherwise. */
	u8 suggestRedStatus;
	/** Alignment for next data element. */
	u8 reservedAlign[2];
	/** OID of the data populator managed data object. */
	ObjID oid;
} DEObjSuggestPopManagedStatus;


//---------------------------------------------------------------------


#pragma pack()


#ifdef __cplusplus
}
#endif


#endif //  __DCDATAEVENT_H_INCLUDED__

