
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Inc. All Rights Reserved.
// 
//  Abstract/Purpose:
//    Data Engine Service/Daemon header file
//
//  Notes:
//
//  Environment:
//    Linux, NetWare Windows NT/2000/2003/2008
//
//  Created/Version:
//    2005 Mar 23 / $Revision: $
//
//  Last Modified By/On:
//    $Author: $ / $Date: $
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCDESVC_H_INCLUDED__
#define __DCDESVC_H_INCLUDED__


//---------------------------------------------------------------------
// Dependent headers
//---------------------------------------------------------------------
#include <dcdetypes.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Definitions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// DESvc event types (evtType)
//---------------------------------------------------------------------

/** DESvc event type: None \n
 *  evtData: None */
#define DESVC_EVT_NONE						(0)

/** DESvc event type: System suspending operation \n
 *  evtData: None */
#define DESVC_EVT_POWER_SUSPEND				(1)

/** DESvc event type: System resuming operation \n
 *  evtData: None */
#define DESVC_EVT_POWER_RESUME				(2)

/** DESvc event type: Device added to system \n
 *  evtData: DESvcDeviceEvtData */
#define DESVC_EVT_DEVICE_ADD				(3)

/** DESvc event type: Device removed from system \n
 *  evtData: DESvcDeviceEvtData */
#define DESVC_EVT_DEVICE_REMOVE				(4)


//---------------------------------------------------------------------
// DESvc device event device types (DESvcDeviceEvtData.devType)
//---------------------------------------------------------------------

/** DESvc device event device type: PCI */
#define DESVC_DEVTYPE_PCI					(1)

/** DESvc device event device type: USB */
#define DESVC_DEVTYPE_USB					(2)

/** DESvc device event device type: Virtual NIC */
#define DESVC_DEVTYPE_VIRNIC				(3)

/** DESvc device event device type: PCMCIA */
#define DESVC_DEVTYPE_PCMCIA				(4)

/** DESvc device event device type: Display:Monitor */
#define DESVC_DEVTYPE_DISPLAY				(5)

/** DESvc device event device type: Battery */
#define DESVC_DEVTYPE_BATTERY				(6)

//---------------------------------------------------------------------
// DESvc event file definitions
//---------------------------------------------------------------------

/** DE variable data subdirectory where event files are located */
#ifdef SYSTEM_TYPE_LINUX
#define DESVC_EVTFILE_SUBDIR				"/log/"
#else
#define DESVC_EVTFILE_SUBDIR				"\\log\\"
#endif

/** DESvc device add event file name */
#define DESVC_EVTFILE_FILENAME_DEVADD		"deviceadd.evt"

/** DESvc device remove event file name */
#define DESVC_EVTFILE_FILENAME_DEVREMOVE	"deviceremove.evt"

/** Maximum size in bytes of line in DESvc event file */
#define DESVC_EVTFILE_MAX_EVTLINE_SIZE		(256)


//---------------------------------------------------------------------
// Structures
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** 
 * \a DESvcDeviceEvtData represents the event data
 * for DESvc device events that are passed to the module's
 * DESvc event handler.
 */
//---------------------------------------------------------------------
typedef struct _DESvcDeviceEvtData
{
	/** Device type (DESVC_DEVTYPE_*) */
	u32 devType;

	/** Device info */
	union _DESvcDevInfo
	{
		/** PCI device info */
		struct _DESvcPCIInfo
		{
			u32 VendorID;
			u32 DeviceID;
			u32 SubVendorID;
			u32 SubDeviceID;
		} PCIInfo;

		/** USB device info */
		struct _DESvcUSBInfo
		{
			u32 VendorID;
			u32 ProductID;
		} USBInfo;
	} Info;
} DESvcDeviceEvtData;


//---------------------------------------------------------------------
// Function Prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Dependent external functions that must be implemented by the host
//---------------------------------------------------------------------

/**
 * ModuleSvcAttach is called to perform initialization of the host.
 *
 * @param argc [in] number of arguments.
 *
 * @param argv [in] array of pointers to arguments.
 *
 * @return Return TRUE if initialization is successful or
 *		FALSE if there is a fatal error.
 */
booln ModuleSvcAttach(_IN s32 argc,
					  _IN astring **argv);

/**
 * ModuleSvcDetach is called to perform cleanup of the host.
 *
 * @param none
 *
 * @return Return TRUE if cleanup is successful or
 *		FALSE if there is a fatal error.
 */
booln ModuleSvcDetach(void);

/**
 * ModuleSvcGetUTF8Name is called to get the service name of the host.
 *
 * @param none
 *
 * @return Return pointer to service name.
 */
const astring *ModuleSvcGetUTF8Name(void);

/**
 * ModuleSvcGetVarDataPath is called to get the variable data path
 * of the host.
 *
 * @param pBufPath [out] buffer allocated by caller for path
 *
 * @param pBufSize [inout] size of buffer
 *
 * @return Return SM_STATUS_SUCCESS if successful or
 *		other status code from dcsmstatus.h.
 */
s32 ModuleSvcGetVarDataPath(_OUT astring *pBufPath,
							_INOUT u32 *pBufSize);

/** 
 * Module DESvc event handler function prototype.
 *
 * @param evtType [in] DESvc event type.
 *
 * @param evtData [in] DESvc event-specific data.
 *
 * @return Return SM_STATUS_SUCCESS if successful or
 *		other status code from dcsmstatus.h.
 */
typedef s32 (__cdecl *PFNDESVCEVTHANDLER)(_IN u32 evtType,
										  _IN void *evtData);

/**
 * ModuleSvcGetDESvcEvtHandler is called to get the address
 * of the function that handles DESvc events.
 * 
 * If the host is not interested in DESvc events, it should return NULL.
 *
 * @param none
 *
 * @return Return address of function to handle DESvc events.
 */
PFNDESVCEVTHANDLER ModuleSvcGetDESvcEvtHandler(void);


//---------------------------------------------------------------------
// Service Library APIs
//---------------------------------------------------------------------

/** 
 * Service Library Attach API function pointer
 *
 * The Attach API is called by the module responsible for loading
 * the Service Library.  The Attach API is called to request the
 * Service Library to initialize and enter the running state after
 * the Service Library is loaded.
 *
 * @note This should only be used as a prototype. The actual exported
 * function name has the form MODULE_SMSvcLibAttach. \n
 * MODULE_SMSvcLibAttach is set up for service library compilation in
 * the following way: \n
 * NT: Set preprocessor value of SVCLIBEXPORT_ATTACHFN to applicable MODULENN_SMSvcLibAttach value. \n
 * LX: Set preprocessor value of SVCLIBEXPORT_ATTACHFN to applicable MODULE_SMSvcLibAttach value. \n
 * Example: If module name is samsvc, the exported function is
 * SAMSVC32_SMSvcLibAttach for NT and SAMSVC_SMSvcLibAttach for LX. \n
 *
 * @return Returns TRUE on success and FALSE on failure.
 *
 * @see FPROCSMSVCLIBDETACH
 */
typedef booln (_cdecl *FPROCSMSVCLIBATTACH)(void);

/**
 * Service Library Attach API name suffix used to form the exported API name
 */
#define SMSVCLIB_APINAME_SUFFIX_ATTACH		"_SMSvcLibAttach"

/** 
 * Service Library Detach API function pointer
 *
 * The Detach API is called by the module that loaded the
 * Service Library.  The Detach API is called to request the
 * Service Library to shut down and clean up before the Service
 * Library is unloaded.
 *
 * @note This should only be used as a prototype. The actual exported
 * function name has the form MODULE_SMSvcLibDetach. \n
 * MODULE_SMSvcLibDetach is set up for service library compilation in
 * the following way: \n
 * NT: Set preprocessor value of SVCLIBEXPORT_DETACHFN to applicable MODULENN_SMSvcLibDetach value. \n
 * LX: Set preprocessor value of SVCLIBEXPORT_DETACHFN to applicable MODULE_SMSvcLibDetach value. \n
 * Example: If module name is samsvc, the exported function is
 * SAMSVC32_SMSvcLibDetach for NT and SAMSVC_SMSvcLibDetach for LX. \n
 *
 * @return Returns TRUE on success and FALSE on failure.
 *
 * @see FPROCSMSVCLIBATTACH
 */
typedef booln (_cdecl *FPROCSMSVCLIBDETACH)(void);

/**
 * Service Library Detach API name suffix used to form the exported API name
 */
#define SMSVCLIB_APINAME_SUFFIX_DETACH		"_SMSvcLibDetach"


//---------------------------------------------------------------------
// Header Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCDESVC_H_INCLUDED__

