
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    Revision: 1
//
//  Last Modified By / On:
//    Author: Ryan Brown / Date: 2012 Jul 25
//
//  Author:
//    Data Manager
//
//  Environment:
//    Linux / Windows
//
//  User/Kernel Mode:
//    User
//
//  Abstract/Purpose:
//    Data Manager multi-langugage Message Registry library interface.
//    The registry provides a mapping from a unique identifier with 
//    optional keys to a set of UTF-8 encoded strings. 
//
//  See Also:
//    dcmsgregdef.h
//
//  Notes:
//    For more information refer to Data Manager Message Registry HLD.
//
//--------------------------------------------------------------------

#ifndef __DCMSGREGAPI_H_INCLUDED__
#define __DCMSGREGAPI_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------

// DE headers
#include "dcossdk.h"
#include "dcdetypes.h"

// local headers
#include "dcmsgregdef.h"


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
//  Function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Null-Separated Value String List and Object
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Free the resources used by the null-separated value string object
 * using the function NSVOInit.
 *
 * @param pnsvo [inout,required] The object buffer.
 *
 * @param initialsize [in,optional] The initial size of the list
 *     buffer. A default size is used when 0 is specified.
 *
 * @return SM_STATUS_SUCCESS on sucess.
 *
 * @return SM_STATUS_UNSUCCESSFUL on error or out of memory.
 *
 * @sa NSVOInit.
 *
 */
SMINTF void _cdecl NSVOUninit(_INOUT NSVObj *pnsvo);


//---------------------------------------------------------------------
/**
 * Initialize the null-separatd value string object. Use NSVOUninit
 * to free resources created by this function. 
 *
 * @param pnsvo [inout,required] The object buffer.
 *
 * @param initialsize [in,optional] The initial size of the list
 *     buffer. A default size is used when 0 is specified.
 *
 * @return SM_STATUS_SUCCESS on sucess.
 *
 * @return SM_STATUS_UNSUCCESSFUL on error or out of memory.
 *
 * @sa NSVOUninit.
 *
 */
SMINTF s32 _cdecl NSVOInit(_INOUT NSVObj *pnsvo,
                           _IN u32 initialsize);


//---------------------------------------------------------------------
/**
 * Add/Append a null-terminated string value into the null-separated
 * value string list. An example usage:
 *     NSVObj nsv;
 *     NSVOInit(&nsv, 256); // add check for return status
 *     NSVOCat(&nsv, "fan1"); // add check for return value
 *     NSVOCat(&nsv, "fan2"); // add check for return value
 *     nsvastring *palist = nsv.pstrvalue; // get the string list
 *     // use palist, in memory looks like "fan1\0fan2\0\0"
 *     NSVOUninit(&nsv);
 *
 * @param pnsvo [inout,required] The object buffer.
 *
 * @param pstrz [in,required] The null-terminated string to be 
 *     appended to the list.
 *
 * @return NULL on error or out of memory.
 *
 * @return A copy of the new null-separated value list string.
 *
 * @sa NSVOInit, NSVOUninit, NSVO_ISNOTEMPTY
 *
 */
SMINTF nsvastring * _cdecl NSVOCat(_INOUT NSVObj *pnsvo,
                                   _IN const astring *pstrz);


//---------------------------------------------------------------------
/**
 * Retrieve Message Registry record information based on ID and Locale.
 *
 * @param pMsgID [in,required] The unique message identifier.
 *
 * @param pLocale [in,required] The language code in ISO 639-1 format.
 *     For example, "en" for English and "zh" for Chinese
 *
 * @param pArgs [in,optional] A list of strings used to substitute
 *     dynamic message arguments. The format is NULL separated value.
 *     For example in memory, 'fan1\0fan2\0fan3\0\0'
 *     Refer to NSVO set of functions to help in the list generation.
 *
 * @param pMsgContent [inout,required] The message registry record
 *     information.
 *
 * @return SM_STATUS_SUCCESS on success.
 *
 * @return SM_STATUS_KEY_NOT_FOUND when record is not found.
 *
 * @return SM_STATUS_LOCALE_DEFAULT when record is found but the
 *     requested Locale is not available and default Locale is used.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 * @sa MsgRegGetContentWNByMsgID
 *
 */
SMINTF s32 _cdecl MsgRegGetContentByMsgID(_IN const astring* pMsgID,
                                          _IN const astring* pLocale,
                                          _IN const nsvastring* pArgs,
                                          _INOUT MessageContent* pMsgContent
                                          );


//---------------------------------------------------------------------
/**
 * Retrieve Message Registry record information based on ID and Locale.
 * This is similar to MsgRegGetContentByMsgID except that the output
 * object contains the numeric conversion of severity and category.
 * Refer to MessageContentWN definition.
 *
 * @param pMsgID [in,required] The unique message identifier.
 *
 * @param pLocale [in,required] The language code in ISO 639-1 format.
 *     For example, "en" for English and "zh" for Chinese
 *
 * @param pArgs [in,optional] A list of strings used to substitute
 *     dynamic message arguments. The format is NULL separated value.
 *     For example in memory, 'fan1\0fan2\0fan3\0\0'
 *     Refer to NSVO set of functions to help in the list generation.
 *
 * @param pMsgContent [inout,required] The message registry record
 *     information.
 *
 * @return SM_STATUS_SUCCESS on success.
 *
 * @return SM_STATUS_KEY_NOT_FOUND when record is not found.
 *
 * @return SM_STATUS_LOCALE_DEFAULT when record is found but the
 *     requested Locale is not available and default Locale is used.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 * @sa MsgRegGetContentByMsgID
 *
 */
SMINTF s32 _cdecl MsgRegGetContentWNByMsgID(_IN const astring* pMsgID,
                                            _IN const astring* pLocale,
                                            _IN const nsvastring* pArgs,
                                            _INOUT MessageContentWN* pMsgContentWN
                                            );


//---------------------------------------------------------------------
/**
 * Retrieve Message Registry message string with substituted arguments
 * based on ID and Locale.
 *
 * @param pMsgID [in,required] The unique message identifier.
 *
 * @param pLocale [in,required] The language code in ISO 639-1 format.
 *     For example, "en" for English and "zh" for Chinese
 *
 * @param pArgs [in,optional] A list of strings used to substitute
 *     dynamic message arguments. The format is NULL separated value.
 *     For example in memory, 'fan1\0fan2\0fan3\0\0'
 *     Refer to NSVO set of functions to help in the list generation.
 *
 * @param pMessage [inout,required] The message registry message
 *     string with substituted arguments when pArgs is present.
 *
 * @param msgSize [in,required] The output string buffer size.
 *
 * @return SM_STATUS_SUCCESS on success.
 *
 * @return SM_STATUS_KEY_NOT_FOUND when record is not found.
 *
 * @return SM_STATUS_LOCALE_DEFAULT when record is found but the
 *     requested Locale is not available and default Locale is used.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 */
SMINTF s32 _cdecl MsgRegGetMessageByMsgID(_IN const astring* pMsgID, 
                                          _IN const astring* pLocale,
                                          _IN const astring* pArgs, 
                                          _INOUT astring* pMessage,
                                          _IN u32 msgSize
                                          );


//---------------------------------------------------------------------
/**
 * Retrieve Message Registry Sub-Category textual representation based
 * on Sub-Category ID and Locale.
 *
 * @param pSubCategoryID [in,required] The unique Sub-Category 
 *     identifier.
 *
 * @param pLocale [in,required] The language code in ISO 639-1 format.
 *     For example, "en" for English and "zh" for Chinese
 *
 * @param pSubCategory [inout,required] The message registry 
 *     Sub-Category string.
 *
 * @param subCategorySize [in,required] The output string buffer size.
 *
 * @return SM_STATUS_SUCCESS on success.
 *
 * @return SM_STATUS_KEY_NOT_FOUND when record is not found.
 *
 * @return SM_STATUS_LOCALE_DEFAULT when record is found but the
 *     requested Locale is not available and default Locale is used.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 */
SMINTF s32 _cdecl MsgRegGetSubCategoryStr(_IN const astring* pSubCategoryID,
                                          _IN const astring* pLocale,
                                          _INOUT astring* pSubCategory,
                                          _IN u32 subCategorySize
                                          );


//---------------------------------------------------------------------
/**
 * Retrieve Message Registry Category textual representation based
 * on Category ID and Locale.
 *
 * @param category [in,required] The unique Category identifier.
 *
 * @param pLocale [in,required] The language code in ISO 639-1 format.
 *     For example, "en" for English and "zh" for Chinese
 *
 * @param pCategory [inout,required] The message registry 
 *     Category string.
 *
 * @param categorySize [in,required] The output string buffer size.
 *
 * @return SM_STATUS_SUCCESS on success.
 *
 * @return SM_STATUS_KEY_NOT_FOUND when record is not found.
 *
 * @return SM_STATUS_LOCALE_DEFAULT when record is found but the
 *     requested Locale is not available and default Locale is used.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 */
SMINTF s32 _cdecl MsgRegGetCategoryStr(_IN s32 category, 
                                       _IN const astring* pLocale, 
                                       _INOUT astring* pCategory, 
                                       _IN u32 categorySize
                                       );


//---------------------------------------------------------------------
/**
 * Retrieve Message Registry Severity textual representation based
 * on Severity ID and Locale.
 *
 * @param severity [in,required] The unique Severity identifier.
 *
 * @param pLocale [in,required] The language code in ISO 639-1 format.
 *     For example, "en" for English and "zh" for Chinese
 *
 * @param pSeverity [inout,required] The message registry 
 *     Severity string.
 *
 * @param severitySize [in,required] The output string buffer size.
 *
 * @return SM_STATUS_SUCCESS on success.
 *
 * @return SM_STATUS_KEY_NOT_FOUND when record is not found.
 *
 * @return SM_STATUS_LOCALE_DEFAULT when record is found but the
 *     requested Locale is not available and default Locale is used.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 */
SMINTF s32 _cdecl MsgRegGetSeverityStr(_IN s32 severity, 
                                       _IN const astring* pLocale, 
                                       _INOUT astring* pSeverity, 
                                       _IN u32 severitySize
                                       );


//---------------------------------------------------------------------
/**
 * Retrieve Message Registry String mapping textual representation 
 * based on a unique ID, Context and Locale.
 *
 * @param id [in,required] The unique mapping identifier.
 *
 * @param pLocale [in,required] The language code in ISO 639-1 format.
 *     For example, "en" for English and "zh" for Chinese
 *
 * @param context [in,required] The unique Context identifier.
 *
 * @param pErrorStr [inout,required] The message registry 
 *     Error string.
 *
 * @param errorStrSize [in,required] The output string buffer size.
 *
 * @return SM_STATUS_SUCCESS on success.
 *
 * @return SM_STATUS_KEY_NOT_FOUND when record is not found.
 *
 * @return SM_STATUS_LOCALE_DEFAULT when record is found but the
 *     requested Locale is not available and default Locale is used.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 */
SMINTF s32 _cdecl MsgRegGetStringByContextID(_IN s32 id,
                                             _IN const astring* pLocale,
                                             _IN s32 context,
                                             _INOUT astring* pErrorStr, 
                                             _IN u32 errorStrSize
                                             );


//---------------------------------------------------------------------
// End
//---------------------------------------------------------------------

#ifdef __cplusplus
}
#endif


#endif //__DCMSGREGAPI_H_INCLUDED__
