
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2010 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Header file for SMIL API definitions and structures
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCSMILDEF_H_INCLUDED__
#define __DCSMILDEF_H_INCLUDED__


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcdetypes.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Defines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Default input output buffer sizes (maximum) */
//---------------------------------------------------------------------
#define DEFAULT_SMIL_IO_SIZE				(4096)

//---------------------------------------------------------------------
/** Minimum input buffer size */
//---------------------------------------------------------------------
#define MIN_SMIL_IN_SIZE					(DEFAULT_SMIL_IO_SIZE)

//---------------------------------------------------------------------
/** Minimum output buffer size */
//---------------------------------------------------------------------
#define MIN_SMIL_OUT_SIZE					(DEFAULT_SMIL_IO_SIZE)

//---------------------------------------------------------------------
/** Minimum number of dispatches in queue (depth) */
//---------------------------------------------------------------------
#define MIN_SMIL_DISPATCH_QUEUE_DEPTH		(1)

//---------------------------------------------------------------------
/** Minimum number of events in queue (depth) */
//---------------------------------------------------------------------
#define MIN_SMIL_EVENT_QUEUE_DEPTH			(256)

//---------------------------------------------------------------------
/** Time in seconds to wait for a response (output) on a dispatch */
//---------------------------------------------------------------------
#ifdef SYSTEM_TYPE_LINUX
#define TIMEOUT_SECONDS_SMIL_DISPATCH_WAIT	(60)
#else
#define TIMEOUT_SECONDS_SMIL_DISPATCH_WAIT	(10)
#endif

//---------------------------------------------------------------------
/** Minimum Data Consumer ID assigned */
//---------------------------------------------------------------------
#define MIN_SMIL_DATA_CONSUMER_ID			(1)

//---------------------------------------------------------------------
/** Maximum Data Consumer ID assigned */
//---------------------------------------------------------------------
#define MAX_SMIL_DATA_CONSUMERS				(254)

//---------------------------------------------------------------------
/** Invalid Data Consumer ID */
//---------------------------------------------------------------------
#define SMIL_INVALID_DATA_CONSUMER_ID		(0)

//---------------------------------------------------------------------
/** Maximum number of connections value that indicates pipe is disabled */
//---------------------------------------------------------------------
#define SMIL_MAX_CONNECTIONS_DISABLED		(0)

//---------------------------------------------------------------------
/** Maximum number of connections allowed on Administrator pipe (0 = pipe disabled) */
//---------------------------------------------------------------------
#define SMIL_MAX_CONNECTIONS_ADMIN			(50)

//---------------------------------------------------------------------
/** Maximum number of connections allowed on Power User pipe (0 = pipe disabled) */
//---------------------------------------------------------------------
#define SMIL_MAX_CONNECTIONS_POWERUSER		(50)

//---------------------------------------------------------------------
/** Maximum number of connections allowed on User pipe (0 = pipe disabled) */
//---------------------------------------------------------------------
#define SMIL_MAX_CONNECTIONS_USER			(50)


//---------------------------------------------------------------------
/** Sizes of server config possible */
//---------------------------------------------------------------------
//---------------------------------------------------------------------
/** Size of SMServerCfg used in current implementation of SMIL */
//---------------------------------------------------------------------
#define SMIL_SMSERVERCFG_SIZE_CURR			(sizeof(SMServerCfg))

//---------------------------------------------------------------------
/** Size of SMServerCfg used in second implementation of SMIL */
//---------------------------------------------------------------------
#define SMIL_SMSERVERCFG_SIZE_ORIG_2		(SMIL_SMSERVERCFG_SIZE_CURR - \
											 sizeof(u32) - \
											 sizeof(u32) - \
											 sizeof(u32))

//---------------------------------------------------------------------
// NOTE:Keep this updated as the Array Manager group fails to update
// their copy of the SDK during their builds and are dependent 
// on this structure
//---------------------------------------------------------------------
//---------------------------------------------------------------------
/** Size of SMServerCfg used in orginal implementation of SMIL */
//---------------------------------------------------------------------
#define SMIL_SMSERVERCFG_SIZE_ORIG			(SMIL_SMSERVERCFG_SIZE_ORIG_2 - \
											 sizeof(u32) - \
											 sizeof(u32))


//---------------------------------------------------------------------
// SMIL Access Types
//---------------------------------------------------------------------

/** SMIL Access Type: Read */
#define SMIL_ACCESSTYPE_READ				BIT(29)

/** SMIL Access Type: Write */
#define SMIL_ACCESSTYPE_WRITE				BIT(30)

/** SMIL Access Type: Admin */
#define SMIL_ACCESSTYPE_ADMIN				BIT(31)


//---------------------------------------------------------------------
// SMIL Privilege Levels
//---------------------------------------------------------------------

/** SMIL Privilege Level: None (no access) */
#define SMIL_PRIVLEVEL_NONE					(0)

/** SMIL Privilege Level: Normal user (read-only access) */
#define SMIL_PRIVLEVEL_USER					(SMIL_ACCESSTYPE_READ)

/** SMIL Privilege Level: Power user (read and write access) */
#define SMIL_PRIVLEVEL_POWERUSER			(SMIL_ACCESSTYPE_READ  | \
											 SMIL_ACCESSTYPE_WRITE)

/** SMIL Privilege Level: Administrator (read, write  and admin access) */
#define SMIL_PRIVLEVEL_ADMINISTRATOR		(SMIL_ACCESSTYPE_READ  | \
											 SMIL_ACCESSTYPE_WRITE | \
											 SMIL_ACCESSTYPE_ADMIN)

/** SMIL Privilege Level: Mask */
#define SMIL_PRIVLEVEL_MASK					(SMIL_ACCESSTYPE_READ  | \
											 SMIL_ACCESSTYPE_WRITE | \
											 SMIL_ACCESSTYPE_ADMIN)

/** Macro to set SMIL Privilege Level in request type */
#define SMIL_REQTYPE_PRIVLEVEL_SET(R,P)		(R | ((~P) & SMIL_PRIVLEVEL_MASK))

/** Macro to get SMIL Privilege Level in request type */
#define SMIL_REQTYPE_PRIVLEVEL_GET(R)		((R & SMIL_PRIVLEVEL_MASK) ^ SMIL_PRIVLEVEL_MASK)

/** Macro to clear SMIL Privilege Level in request type */
#define SMIL_REQTYPE_PRIVLEVEL_CLEAR(R)		(R & (~SMIL_PRIVLEVEL_MASK))


//---------------------------------------------------------------------
// Structures
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// NOTE: Need to pack on byte boundary as Legacy Consumers expect it
// to be on byte boundaries
//---------------------------------------------------------------------
#pragma pack(1)


//---------------------------------------------------------------------
// SMServerCfg
/** 
 * Structure used to setup and report SMIL consumer/manager connections.
 *
 * Data structure for server side configuration specified by Data Manager
 * on SMServerAttach.
 *
 * @see SMServerAttach, SMClientGetServerCfg
 */
//---------------------------------------------------------------------
typedef struct _SMServerCfg
{
	/** Minimum depth allocated by the Request and Response Queue. */
	u32 minDispatchQueueDepth;
	/** Maximum size allocated for individual Request data buffer in the Request Queue. */
	u32 inBufAllocSize;
	/** Maximum size allocated for individual Response data buffer in the Response Queue. */
	u32 outBufAllocSize;
	/** Maximum size allocated for individual Data Event buffer in the Event Queue. */
	u32 evtBufAllocSize;
	/** Minium queue depth of the event block queue to be allocated by data consumer. Q size = (evtBufAllocSize * maxEvtQDepth). */
	u32 minEvtQueueDepth;
	/** The time in seconds for which the API method SMClientDispatch waits after a Request for a Response to be generated by the Data Manager. */
	s32 dispatchTimeOut;
	/** Minimum value (inclusive) to be used for data consumer identifiers with the exception of SMIL_INVALID_DATA_CONSUMER_ID. */
	u32 minDataConsumerID;
	/** Maximum number of data consumers that should be allowed to attach. */
	u32 maxDataConsumers;
	/** Maximum number of connections allowed on Administrator pipe (0 = pipe disabled). */
	u32 maxConnectionsAdmin;
	/** Maximum number of connections allowed on Power User pipe (0 = pipe disabled). */
	u32 maxConnectionsPowerUser;
	/** Maximum number of connections allowed on User pipe (0 = pipe disabled). */
	u32 maxConnectionsUser;
} SMServerCfg;


#pragma pack()


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCSMILDEF_H_INCLUDED__

