
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2010 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Dell SM Debugging Aid API
//
//  See Also:
//    Data Engine High Level Design document
//
//  Notes:
//    Make sure to link the module with multithreaded C runtime library
//    when using with threads.
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCSYSDEBUG_H_INCLUDED__
#define __DCSYSDEBUG_H_INCLUDED__


//---------------------------------------------------------------------
// Include files that this module is dependent on
//---------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>


#ifdef SYSTEM_TYPE_LINUX
#include <bits/wordsize.h>
#endif // SYSTEM_TYPE_LINUX


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Definitions for debugging macros
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// USAGE NOTES:
//
// Define the following sybols BEFORE including this file:
// - Use SYSTEM_TYPE_XXX to define the OS environment intended for this
// module
//
// - Use SYSTEM_DEBUG_LEVEL=X where X is defined below to define intent 
// to include trace macros
// In _DEBUG compile all message macros as compiled and SysDbgX level
// macros matching the default level specified
// In release mode these macros are not compiled unless specified
// as shown below.
//
// - Use the following guidelines to decide SysDbg or SysDbgMsg level:
// SYSTEM_DEBUG_LEVEL  = 0 turns off debug tool and no code is included.
// SYSTEM_DEBUG_LEVEL  = 1 is reserved for future use.
// SYSTEM_DEBUG_LEVEL  = 2 is high priority messages. Example: System Abort/Abend imminent.
// SYSTEM_DEBUG_LEVEL  = 3 is medium priority messages. Example: Malloc Failed/Pointer is NULL.
// SYSTEM_DEBUG_LEVEL  = 4 is low priority messages. Example: The value of varX is %d/Enter func X./Invalid Value of X.
// If it is a message for unit debug purposes it will most likely be at level 4.
// If it is a message for field debug purposes it will most likely be at level 3.
// If it is a critical message for field debug purposes it will most likely be at level 2.
//
// The default destination is file defined by DEFAULT_SYSDBG_FILENAME in system/current directory.
//
// SYS_DBG_DECLARE_GLOBAL_CODEDATA to setup global code and data needed by debug tool.
// Use SysDbgN(X) for debug data and code X at level N. Note Single Rounded Brackets.
// Use SysDbgMsgN((X)) for debug message X at level N. Note Double Rounded Brakets.
// Use SysDbgDumpBufN((X,Y)) for debug dump buffer X of length Y at level N. Note Double Rounded Brakets.
// Use SysDbgAssertN(X) for inserting assertion or checking code. Note Single Rounded Brackets.
// If an always on macro is needed use SysDbg versions without the N postfix.
//
// The following are always on macros
// Use SysDbgInit(dbgTarget, dbgFileName) to override default settings.
// Use SysDbgClearLog() to clear log on disk.
// 
//
// - Use SYSTEM_DEBUG_ENABLE_IN_RELEASE to enable trace message compilation in release builds.
// - Use SYSTEM_DEBUG_MEM_USAGE to check for memory leaks and low memory
// simulation conditions on the application
//
//---------------------------------------------------------------------


// The user can override the _DEBUG project default debug message
// level in order to reduce noise
// so the specified level precedes following defines
// additionally we need to compile in all trace macros
// regardless of the level so that release and debug
// code typically contain all macros

#ifdef SYSTEM_DEBUG_LEVEL

// check reserved use
#if (SYSTEM_DEBUG_LEVEL == 1)
#error SYSTEM_DEBUG_LEVEL specified is reserved.
#endif

#endif // SYSTEM_DEBUG_LEVEL


#ifdef _DEBUG

// in the debug version we use the level specifed
// in the module where SYS_DBG_DECLARE_GLOBAL_CODEDATA
// for the initial level of trace macros
// is defined so that that is applied consistently
#ifdef SYSTEM_DEBUG_LEVEL

#if (SYSTEM_DEBUG_LEVEL == 2)
#define SYSTEM_DEBUG_LEVEL_DEFAULT	(2)
#elif (SYSTEM_DEBUG_LEVEL == 3)
#define SYSTEM_DEBUG_LEVEL_DEFAULT	(3)
#elif (SYSTEM_DEBUG_LEVEL == 4)
#define SYSTEM_DEBUG_LEVEL_DEFAULT	(4)
#endif

#else // SYSTEM_DEBUG_LEVEL

// else we default to all on
#define SYSTEM_DEBUG_LEVEL_DEFAULT	(4)

#endif // SYSTEM_DEBUG_LEVEL

// compile in all trace macros
#undef SYSTEM_DEBUG_LEVEL
#define SYSTEM_DEBUG_LEVEL			(4)

#else // _DEBUG

// make sure we are not accidently including macros
#ifdef SYSTEM_DEBUG_ENABLE_IN_RELEASE

// in release version we turn the initial level
// of trace macros off
// however we compile in all trace macros
// as in the case of debug
#define SYSTEM_DEBUG_LEVEL_DEFAULT	(0)

// compile in all trace macros
#undef SYSTEM_DEBUG_LEVEL
#define SYSTEM_DEBUG_LEVEL			(4)

#else // SYSTEM_DEBUG_ENABLE_IN_RELEASE

#if (SYSTEM_DEBUG_LEVEL > 0)
#error System debug trace tool turned on for non-debug build.
#endif // (SYSTEM_DEBUG_LEVEL > 0)

// make sure we do not accidently include any trace macros
#undef SYSTEM_DEBUG_LEVEL
//---------------------------------------------------------------------
/** Determines verbosity level of trace logging.
 * Use the following guidelines to decide SysDbgX or SysDbgMsgX or 
 * SysDbgAssertX level:
 * SYSTEM_DEBUG_LEVEL  = 0 turns off debug tool and no code is included.
 * SYSTEM_DEBUG_LEVEL  = 1 is reserved for future use.
 * SYSTEM_DEBUG_LEVEL  = 2 is high priority messages. Example: System Abort/Abend imminent.
 * SYSTEM_DEBUG_LEVEL  = 3 is medium priority messages. Example: Malloc Failed/Pointer is NULL.
 * SYSTEM_DEBUG_LEVEL  = 4 is low priority messages. Example: The value of varX is %d/Enter func X./Invalid Value of X.
 * If it is a message for unit debug purposes it will most likely be at level 4.
 * If it is a message for field debug purposes it will most likely be at level 3.
 * If it is a critical message for field debug purposes it will most likely be at level 2.
 */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_LEVEL			(0)

#endif // SYSTEM_DEBUG_ENABLE_IN_RELEASE

#endif // _DEBUG

// determine processor data width
#ifdef SYSTEM_TYPE_WINNT
#if defined (_M_IA64) || defined (_M_AMD64)
// IA-64 or AMD-64 architecture
#define SYSTEM_DEBUG_DATA_WIDTH		(64)
#endif
#ifdef _M_IX86
// IA-32 architecture
#define SYSTEM_DEBUG_DATA_WIDTH		(32)
#endif
#endif

#ifdef SYSTEM_TYPE_NETWARE
#define SYSTEM_DEBUG_DATA_WIDTH		(32)
#endif

#ifdef SYSTEM_TYPE_LINUX
#define SYSTEM_DEBUG_DATA_WIDTH		(__WORDSIZE)
#endif

#ifdef SYSTEM_TYPE_DOS
#define SYSTEM_DEBUG_DATA_WIDTH		(16)
#endif


//---------------------------------------------------------------------
// Destination where messages sent
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Log destination to nul, no trace log output */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_NONE			(-1)

//---------------------------------------------------------------------
/** Log destination to file, may be combined with other destination except NONE. */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_FILEv2			(0x00)

//---------------------------------------------------------------------
/** Log destination to file, may be combined with other destination except NONE. */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_FILE			(0x01)

//---------------------------------------------------------------------
/** Log destination to stdout, may be combined with other destination except NONE. */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_STDOUT			(0x02)

//---------------------------------------------------------------------
/** Log destination to stderr, may be combined with other destination except NONE. */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_SCREEN			(0x04)

//---------------------------------------------------------------------
/** Maximum length for user defined dbgFileName */
//---------------------------------------------------------------------
#define SYSDBG_MAX_FILENAME_LEN		(32)

//---------------------------------------------------------------------
/** Maximum length for user defined dbgFileName */
//---------------------------------------------------------------------
#define SYSDBG_MAX_FILEPATH_LEN		(256)


#if (SYSTEM_DEBUG_LEVEL < 1)


// shut off all macros
//---------------------------------------------------------------------
/** Setup global code and data needed by debug tool.
 * This macro declares and instantiates code section with the 
 * functions needed for the debugging trace system. This macro is 
 * typically declared around the main or ModuleAttach method. None of 
 * the debug trace methods can be used without exactly one instance 
 * of the functions being declared in all modules (object) files 
 * linked in. Please note the debug system may not be shareable 
 * amongst dynamically linked libraries and shared objects due to 
 * Microsoft, Novell and GNU C-lib runtime rules.
 */
//---------------------------------------------------------------------
#define SYS_DBG_DECLARE_GLOBAL_CODEDATA

//---------------------------------------------------------------------
/** Override default settings and initializes trace function.
 * If the SysDbgInit function is not called the default destination 
 * is to a file. The default name of the file is 
 * DEFAULT_SYSDBG_FILENAME and depends on a 32-Bit, 64-bit and OS. 
 * The exact location of the file is different according to the OS 
 * type. On windows systems the folder is %SytemRoot%\, On Netware the 
 * folder is SYS:SYSTEM\ directory and on LINUX the folder is 
 * %DebugPath%/var/log/. This function overwrites the default debug 
 * trace output parameters and can be called at any time but is 
 * typically called and set once on startup.
 *
 * @param dbgTarget [in] Specifies the destination(s) device to send 
 * trace output messages to.
 *
 * @param dbgFileName [in] Filename to send debug trace log 
 * output to.
 *
 * @see SYS_DBG_DECLARE_GLOBAL_CODEDATA
 *
 */
//---------------------------------------------------------------------
#define SysDbgInit(dbgTarget, dbgFileName)

//---------------------------------------------------------------------
/** Clear trace log.
 * This function clears the debug trace log output device. Currently, 
 * only the SYSTEM_DEBUG_FILE device is supported.  This will clear 
 * the debug log specified by SysDbgInit or the default log.
 *
 * @see SysDbgInit
 *
 */
//---------------------------------------------------------------------
#define SysDbgClearLog()

//---------------------------------------------------------------------
/** Get current verbosity level. */
//---------------------------------------------------------------------
#define SysDbgGetLevel()

//---------------------------------------------------------------------
/** This function ouputs a trace message to specified output device(s).
 *
 * @param X [in] (fmt, ...) :format string for output. See the printf 
 * format string for your compiler or OS. arguments for the specified 
 * format string.
 *
 * @note Parameter X must contain enclosing parenthesis. 
 * Example: SysDbgMsg(("failed status: %ld\n", status))
 *
 * @see SysDbgInit
 *
 */
//---------------------------------------------------------------------
#define SysDbgMsg(X)

//---------------------------------------------------------------------
/** This function outputs a trace message at specified verbosity.
 *
 * @param Y [in] run time verbosity
 *
 * @param X [in] (fmt, ...) :format string for output. See the printf 
 * format string for your compiler or OS. arguments for the specified 
 * format string.
 *
 * @note Parameter X must contain enclosing parenthesis. 
 * Example: SysDbgMsgAtLevel(3, ("failed status: %ld\n", status))
 *
 * @see SysDbgMsg
 *
 */
//---------------------------------------------------------------------
#define SysDbgMsgAtLevel(Y, X)

//---------------------------------------------------------------------
/** This function ouputs buffer contents to specified output device(s).
 * This function writes a hexadecimal character dump of a memory 
 * buffer contents to the debugging output device(s).
 *
 * @param X [in] (pBuf, len) :pointer to memory buffer to dump 
 * in hexadecimal and character form and the buffer length.
 *
 * @note Parameter X must contain enclosing parenthesis. 
 * Example: SysDbgDumpBuf(("failed status: %ld\n", status))
 *
 * @see SysDbgInit
 *
 */
//---------------------------------------------------------------------
#define SysDbgDumpBuf(X)

//---------------------------------------------------------------------
/** This function ouputs buffer contents at specified verbosity.
 * This function writes a hexadecimal character dump of a memory 
 * buffer contents to the debugging output device(s).
 *
 * @param Y [in] run time verbosity
 *
 * @param X [in] (pBuf, len) :pointer to memory buffer to dump 
 * in hexadecimal and character form and the buffer length.
 *
 * @note Parameter X must contain enclosing parenthesis. 
 * Example: SysDbgDumpBufAtLevel(3, ("failed status: %ld\n", status))
 *
 * @see SysDbgDumpBuf
 *
 */
//---------------------------------------------------------------------
#define SysDbgDumpBufAtLevel(Y, X)

//---------------------------------------------------------------------
/** Used to inject debug assertions into debug compiled code.
 *
 * @param X [in] ({code}) code fragment to to placed in module at
 * specified compile verbosity level
 *
 * @note Parameter X may optionally contain enclosing bracket 
 * depending on desired scoping of variables within. 
 * Example: SysDbg({ assert(pHandle); })
 *
 * @see SYSTEM_DEBUG_LEVEL
 *
 */
//---------------------------------------------------------------------
#define SysDbg(X)

//---------------------------------------------------------------------
/** Used to inject debug assertions into debug compiled code.
 * 
 * @param Y [in] run time verbosity
 *
 * @param X [in] ({code}) code fragment to to placed in module at
 * specified compile verbosity level. Is executed if matches run time
 * verbosity level
 *
 * @see SYSTEM_DEBUG_LEVEL
 *
 */
//---------------------------------------------------------------------
#define SysDbgAtLevel(Y, X)

//---------------------------------------------------------------------
/** Used to insert assertion or checking code.
 *
 * @param X [in] ({code}) code fragment to to placed in module at
 * specified compile verbosity level
 *
 * @note Parameter X may optionally contain enclosing bracket 
 * depending on desired scoping of variables within. 
 * Example: SysDbg({ assert(pHandle); })
 *
 * @see SYSTEM_DEBUG_LEVEL
 *
 */
//---------------------------------------------------------------------
#define SysDbgAssert(X)

//---------------------------------------------------------------------
// Debug feature control flags
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Enables trace message compilation in release builds. */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_ENABLE_IN_RELEASE
#undef SYSTEM_DEBUG_ENABLE_IN_RELEASE

//---------------------------------------------------------------------
/** 
 * Enables check for memory leaks and low memory simulation
 * conditions on the application; It is currently used only with
 * Windows compilation.
 */
//---------------------------------------------------------------------
#define SYSTEM_DEBUG_MEM_USAGE
#undef SYSTEM_DEBUG_MEM_USAGE


#else // (SYSTEM_DEBUG_LEVEL < 1)


// default destination
#ifdef _DEBUG
#define SYSDBG_DESTINATION_DEFAULT	(SYSTEM_DEBUG_FILE)
#else
#define SYSDBG_DESTINATION_DEFAULT	(SYSTEM_DEBUG_NONE)
#endif // _DEBUG

// default log file name
#if (SYSTEM_DEBUG_DATA_WIDTH == 64)
#define DEFAULT_SYSDBG_FILENAME		"dcomsa64.log"
#elif (SYSTEM_DEBUG_DATA_WIDTH == 32)
#define DEFAULT_SYSDBG_FILENAME		"dcomsa32.log"
#elif (SYSTEM_DEBUG_DATA_WIDTH == 16)
#define DEFAULT_SYSDBG_FILENAME		"dcomsa16.log"
#endif

// default path
#ifdef SYSTEM_TYPE_WINNT
#define SYSDBG_PATH_DEFAULT			".\\"
#define SYSDBG_PATH_SEPARATOR		"\\"
#endif

#ifdef SYSTEM_TYPE_NETWARE
#define SYSDBG_PATH_DEFAULT			"sys:system\\"
#define SYSDBG_PATH_SEPARATOR		"\\"
#endif

#ifdef SYSTEM_TYPE_LINUX
#   if !defined(OMLOGDIR)
#       define OMLOGDIR "/var/log/"
#   endif
#define SYSDBG_PATH_DEFAULT			OMLOGDIR
#define SYSDBG_PATH_SEPARATOR		"/"
#endif

#ifdef SYSTEM_TYPE_DOS
#define SYSDBG_PATH_DEFAULT			".\\"
#define SYSDBG_PATH_SEPARATOR		"\\"
#endif


// path environment variable
// thus far has worked usefully only on WINNT
#ifdef SYSTEM_TYPE_WINNT
#define SysDbgGetPathEnv()			getenv("SystemRoot")
#endif

#ifdef SYSTEM_TYPE_NETWARE
#define SysDbgGetPathEnv()			NULL
#endif

#ifdef SYSTEM_TYPE_LINUX
#define SysDbgGetPathEnv()			NULL
#endif

#ifdef SYSTEM_TYPE_DOS
#define SysDbgGetPathEnv()			NULL
#endif

// global variables are not available externally
// code routines are available externally
int __SysDbgGetLevel(void);
int __SysDbgIsLevelEnabled(int macroLevel);
int __SysDbgInit(int dbgTarget, const char *pDbgFileName);
int __SysDbgClearLog(void);
int __SysDbgPrint(const char *fmt, ...);
int __SysDbgvPrint(const char *fmt, va_list vap);
void __SysDbgDumpBuffer(const void *pBuf, unsigned long len);

// The global code data allocator macro
// Begin: SYS_DBG_DECLARE_GLOBAL_CODEDATA
#define SYS_DBG_DECLARE_GLOBAL_CODEDATA									\
/* Start Global Data */													\
																		\
/* Destinations for logging */                                          \
int __sysDbgTarget = SYSDBG_DESTINATION_DEFAULT;						\
/* Debug Level */                                                       \
int __sysDebugLevel = SYSTEM_DEBUG_LEVEL_DEFAULT;						\
/* Log file name on disk */                                             \
char __sysDbgFileName[SYSDBG_MAX_FILENAME_LEN] = DEFAULT_SYSDBG_FILENAME;	\
/* path and file name on disk */                                        \
char __sysDbgPathFileName[SYSDBG_MAX_FILEPATH_LEN] = SYSDBG_PATH_DEFAULT DEFAULT_SYSDBG_FILENAME;	\
																		\
/* End Global Data */                                                   \
																		\
/* Start Global Code */													\
																		\
																		\
/* get current level enabled */											\
int __SysDbgGetLevel(void)												\
{                                                                       \
  	/* return current level */											\
	return __sysDebugLevel;												\
} /* __SysDbgGetLevel */												\
																		\
																		\
/* check if level criteria satisfied */									\
int __SysDbgIsLevelEnabled(int macroLevel)                              \
{                                                                       \
	/* check if sysdbgLevel equal or greater to macro level */			\
	if (__SysDbgGetLevel() >= macroLevel)								\
	{																	\
		/* true */														\
		return 1;														\
	}																	\
																		\
	/* false */															\
	return 0;															\
} /* __SysDbgIsLevelEnabled */	                                        \
																		\
																		\
/* parse a log line for name=value pair */								\
char *__SysDbgGetNVPair(char *pNVPair, char *pName)                     \
{                                                                       \
	char *pNVVal;														\
																		\
	/* parse for '=' sign followed by value */							\
	if ((pNVVal = strpbrk(pNVPair, "=")) == NULL)						\
	{																	\
		return NULL;													\
	}																	\
																		\
	*pNVVal = '\0';														\
																		\
	/* check the name */												\
	if (stricmp(pNVPair, pName) == 0)									\
	{																	\
		*pNVVal = '=';													\
		pNVVal++;														\
		return pNVVal;													\
	}																	\
	*pNVVal = '=';														\
																		\
	return NULL;														\
} /* __SysDbgGetNVPair */												\
																		\
																		\
/* clear log on disk */                                                 \
int __SysDbgClearLog(void)                                              \
{                                                                       \
	FILE *fp;       													\
	int retVal;															\
																		\
	/* only file supported */ 											\
	/* wax the file */ 													\
	if ((fp = fopen(__sysDbgPathFileName, "w")) == NULL)				\
	{																	\
		/* does not exist or could not write */							\
		return -1;														\
	}																	\
																		\
	retVal = fprintf(fp, "loglevel=%d\nlogtarget=%d\nlogpathfile=%s\n", __sysDebugLevel, __sysDbgTarget, __sysDbgPathFileName);	\
																		\
	/* yippee !!! */ 													\
	fclose(fp);															\
	/* deed is done */ 													\
	return retVal;														\
} /* __SysDbgClearLog */		                                        \
																		\
																		\
/* get current level enabled */											\
void __SysDbgGetLevelAndDestinationFromLog(void)						\
{                                                                       \
	FILE *fp;       													\
	char *pVal;															\
	int numLines;     													\
	int ii;		     													\
	int jj;		     													\
	int needClearLog;													\
	char lineBuf[256]; 													\
																		\
																		\
	/* read lines */													\
	numLines = 6;														\
	/* init flags */													\
	needClearLog = 0;													\
																		\
	/* read the first N lines of the file */							\
	if ((fp = fopen(__sysDbgPathFileName, "r")) == NULL)				\
	{																	\
		/* create log and write level, target and path file to log */	\
		__SysDbgClearLog();												\
		return;															\
	}																	\
																		\
	/* read required lines */											\
	for (ii = 0; ii < numLines; ii++)									\
	{																	\
		/* read lines */												\
		if (fgets(lineBuf, sizeof(lineBuf), fp)	!= NULL)				\
		{																\
			/* terminate line */										\
			lineBuf[sizeof(lineBuf) - 1] = '\0';						\
			/* clear hard carriage returns if present */				\
			jj = (int)strlen(lineBuf);									\
			if (jj > 0)													\
			{															\
				jj--;													\
				if (lineBuf[jj] == '\n')								\
				{														\
					lineBuf[jj] = '\0';									\
				}														\
			}															\
																		\
			/* parse line */											\
																		\
			/* look for level */										\
			if ((pVal = __SysDbgGetNVPair(lineBuf, "loglevel")) != NULL)	\
			{															\
				jj = atoi(pVal);										\
				if (jj >= 0)											\
				{														\
					__sysDebugLevel = jj;								\
				}														\
			}															\
			else														\
			{															\
				/* look for destination */								\
				if ((pVal = __SysDbgGetNVPair(lineBuf, "logtarget")) != NULL)	\
				{														\
					/* negative value is valid */						\
					__sysDbgTarget = atoi(pVal);						\
				}														\
				else													\
				{														\
					/* look for path file name */						\
					if ((pVal = __SysDbgGetNVPair(lineBuf, "logpathfile")) != NULL)	\
					{													\
						if (*pVal != '\0')								\
						{												\
							strcpy(__sysDbgPathFileName, pVal);			\
							needClearLog = 1;							\
						}												\
					}													\
				}														\
			}															\
		}																\
	} /* for (ii... */													\
																		\
	/* close file */													\
	fclose(fp);															\
																		\
	/* create the file? */												\
	if (needClearLog == 1)												\
	{																	\
		/* create log and write level, target and path file to log */	\
		if ((fp = fopen(__sysDbgPathFileName, "r")) == NULL)			\
		{																\
			__SysDbgClearLog();											\
		}																\
		else															\
		{																\
			/* close file */											\
			fclose(fp);													\
		}																\
	}																	\
} /* __SysDbgGetLevelAndDestinationFromLog */							\
																		\
																		\
/* override default settings */                                         \
int __SysDbgInit(int dbgTarget, const char *pDbgFileName)				\
{                                                                       \
	char *pSysDbgPath;													\
	int fileNameTooLong = 0;											\
	int pathFromEnvTooLong = 0;											\
																		\
																		\
	/* copy filename */													\
	if ((strlen(pDbgFileName) + 1) <= SYSDBG_MAX_FILENAME_LEN)			\
	{																	\
		strcpy(__sysDbgFileName, pDbgFileName);				 			\
	}																	\
	else																\
	{																	\
		fileNameTooLong = 1;											\
	}																	\
																		\
	/* start with null terminated string and conctenate */ 				\
	__sysDbgPathFileName[0] = '\0';										\
	pSysDbgPath = SysDbgGetPathEnv();									\
	if (pSysDbgPath != NULL)											\
	{																	\
		if ((strlen(pSysDbgPath) + strlen(SYSDBG_PATH_SEPARATOR) + strlen(__sysDbgFileName) + 1) <= SYSDBG_MAX_FILEPATH_LEN)	\
		{																\
			strcat(__sysDbgPathFileName, pSysDbgPath);					\
			strcat(__sysDbgPathFileName, SYSDBG_PATH_SEPARATOR);		\
		}																\
		else															\
		{																\
			strcat(__sysDbgPathFileName, SYSDBG_PATH_DEFAULT);			\
			pathFromEnvTooLong = 1;										\
		}																\
	}																	\
	else																\
	{																	\
		strcat(__sysDbgPathFileName, SYSDBG_PATH_DEFAULT);				\
	}																	\
																		\
	strcat(__sysDbgPathFileName, __sysDbgFileName);						\
																		\
	/* setup destinations */											\
	__sysDbgTarget = dbgTarget;											\
																		\
	/* setup level, destinations and other overrides */					\
	__SysDbgGetLevelAndDestinationFromLog();							\
																		\
	/* print message if filename too long */							\
	if (fileNameTooLong == 1)											\
	{																	\
		__SysDbgPrint("SysDbgInit: input filename too long: %s\n", pDbgFileName);	\
	}																	\
																		\
	/* print message if path from envionment too long */				\
	if (pathFromEnvTooLong == 1)										\
	{																	\
		__SysDbgPrint("SysDbgInit: path from env too long: %s\n", pSysDbgPath);	\
	}																	\
																		\
	return 0;															\
} /* __SysDbgInit */                                                    \
																		\
																		\
/* print debug message to destination */                                \
int __SysDbgPrint(const char *fmt, ...)                                 \
{                                                                       \
	va_list vap;														\
	int retVal;															\
																		\
	/* write message to destination */ 									\
	retVal = 0;															\
																		\
	/* check if log destination is disabled */							\
	if (__sysDbgTarget > SYSTEM_DEBUG_NONE)								\
	{																	\
		if ((__sysDbgTarget == SYSTEM_DEBUG_FILEv2) ||					\
			((__sysDbgTarget & SYSTEM_DEBUG_FILE) != 0))				\
		{																\
			FILE    *fp;												\
																		\
			if ((fp = fopen(__sysDbgPathFileName, "a")) != NULL)		\
			{															\
				va_start(vap, fmt);										\
				retVal = vfprintf(fp, fmt, vap);						\
				va_end(vap);											\
				fclose(fp);												\
			}															\
		}																\
																		\
		if ((__sysDbgTarget & SYSTEM_DEBUG_SCREEN) != 0)				\
		{																\
			va_start(vap, fmt);											\
			retVal = vfprintf(stderr, fmt, vap);						\
			va_end(vap);												\
		}																\
																		\
		if ((__sysDbgTarget & SYSTEM_DEBUG_STDOUT) != 0)				\
		{																\
			va_start(vap, fmt);											\
			retVal = vfprintf(stdout, fmt, vap);						\
			va_end(vap);												\
		}																\
	}																	\
																		\
	return retVal;														\
} /* __SysDbgPrint */                                                   \
																		\
																		\
/* print debug message to destination */                                \
int __SysDbgvPrint(const char *fmt, va_list vap)                        \
{                                                                       \
	int retVal;															\
																		\
	/* write message to destination */ 									\
	retVal = 0;															\
																		\
	/* check if log destination is disabled */							\
	if (__sysDbgTarget > SYSTEM_DEBUG_NONE)								\
	{																	\
		if ((__sysDbgTarget == SYSTEM_DEBUG_FILEv2) ||					\
			((__sysDbgTarget & SYSTEM_DEBUG_FILE) != 0))				\
		{																\
			FILE    *fp;												\
																		\
			if ((fp = fopen(__sysDbgPathFileName, "a")) != NULL)		\
			{															\
				retVal = vfprintf(fp, fmt, vap);						\
				fclose(fp);												\
			}															\
		}																\
																		\
		if ((__sysDbgTarget & SYSTEM_DEBUG_SCREEN) != 0)				\
		{																\
			retVal = vfprintf(stderr, fmt, vap);						\
		}																\
																		\
		if ((__sysDbgTarget & SYSTEM_DEBUG_STDOUT) != 0)				\
		{																\
			retVal = vfprintf(stdout, fmt, vap);						\
		}																\
	}																	\
																		\
	return retVal;														\
} /* __SysDbgvPrint */													\
																		\
																		\
/* dump buffer in debug format */                                       \
void __SysDbgDumpBuffer(const void *pBuf, unsigned long len)			\
{                                                                       \
	const unsigned char *buf = (const unsigned char *)pBuf;				\
	char *hexstr = "0123456789ABCDEF";                                  \
	unsigned long i, j, k;                                              \
	char buffer[80]; /* 80 columns */                                   \
																		\
																		\
	for(i = 0; i < len; i+=16)                                          \
	{                                                                   \
		/* fill buffer with blanks */									\
		memset(buffer, ' ', 78);										\
		/* ASCIIZ termination */										\
		buffer[78] = '\0';												\
		k = (16 < (len - i)) ? 16 : (len - i);							\
		for(j = 0; j < k; j++)											\
		{																\
			buffer[(j*3)] = hexstr[(buf[i + j] >> 4) & 0x0F];			\
			buffer[(j*3) + 1] = hexstr[(buf[i + j]) & 0x0F];			\
			buffer[j + 54] = ((buf[i + j] >= ' ') &&					\
			(buf[i + j] <= 'z')) ? buf[i + j] : '.';					\
			if ((j > 0) && ((j % 4) == 0))								\
			{															\
				buffer[(j*3) - 1] = ':';								\
			}															\
		}																\
		__SysDbgPrint("%s\n", buffer);									\
	}                                                                   \
} /* __SysDbgDumpBuffer */                                              \
/* End Global Code */
// End: SYS_DBG_DECLARE_GLOBAL_CODEDATA


// setup always on macros if this module enabled

#define SysDbgInit(dbgTarget, dbgFileName)		__SysDbgInit(dbgTarget, dbgFileName)

#define SysDbgClearLog()		__SysDbgClearLog()

#define SysDbgGetLevel()		__SysDbgGetLevel()

#define SysDbgMsg(X)			__SysDbgPrint X

#define SysDbgMsgAtLevel(Y, X)	if (__SysDbgIsLevelEnabled(Y) == 1) __SysDbgPrint X

#define SysDbgDumpBuf(X)		__SysDbgDumpBuffer X

#define SysDbgDumpBufAtLevel(Y, X)	if (__SysDbgIsLevelEnabled(Y) == 1) __SysDbgDumpBuffer X

#define SysDbg(X)				X

#define SysDbgAtLevel(Y, X)		if (__SysDbgIsLevelEnabled(Y) == 1) { X }


#endif // (SYSTEM_DEBUG_LEVEL < 1)



// define debugger macros as needed
#ifdef _DEBUG

// AVR (05/11/97): Leave semi-colon responsibilty to caller

#ifdef SYSTEM_TYPE_WINNT
#define SysDebugger()			DebugBreak() // IA-32 and IA-64 safe
#endif

#ifdef SYSTEM_TYPE_NETWARE
#define SysDebugger()			EnterDebugger() // SMP and NW5 safe
#endif

#else // _DEBUG

// We need to make sure that release versions do not
// enter the debugger
//---------------------------------------------------------------------
/** Used to introduce a software breakpoint in code.
 * This function can be used to jump into the debugger. If the 
 * debugger is not available the process will terminate with an 
 * exception (Dr. Watson, ABEND or Segmentation Fault).
 * @note Should as function call. 
 */
//---------------------------------------------------------------------
#define SysDebugger()

#endif // _DEBUG



// define macros that apply to level 2
#if (SYSTEM_DEBUG_LEVEL >= 2)

#if (SYSTEM_DEBUG_LEVEL_DEFAULT >= 2)
#define SysDbg2(X)				SysDbg(X)
#else
#define SysDbg2(X)
#endif
#define SysDbgAssert2(X)		SysDbgAtLevel(2, X)
#define SysDbgMsg2(X)			SysDbgMsgAtLevel(2, X)
#define SysDbgDumpBuf2(X)		SysDbgDumpBufAtLevel(2, X)

#else

//---------------------------------------------------------------------
/** Debug data and code X at compile verbosity >= 2. 
 * @see SysDbg(X)
 */
//---------------------------------------------------------------------
#define SysDbg2(X)

//---------------------------------------------------------------------
/** Debug data and code X at compile and run verbosity >= 2.
 * @see SysDbgAssert(X)
 */
//---------------------------------------------------------------------
#define SysDbgAssert2(X)

//---------------------------------------------------------------------
/** Output message X at compile verbosity >= 2.
 * @see SysDbgMsg(X)
 */
//---------------------------------------------------------------------
#define SysDbgMsg2(X)

//---------------------------------------------------------------------
/** Output buffer X at compile verbosity >= 2. 
 * @see SysDbgDumpBuf(X)
 */
//---------------------------------------------------------------------
#define SysDbgDumpBuf2(X)

#endif // (SYSTEM_DEBUG_LEVEL >= 2)


// define macros that apply to level 3
#if (SYSTEM_DEBUG_LEVEL >= 3)

#if (SYSTEM_DEBUG_LEVEL_DEFAULT >= 3)
#define SysDbg3(X)				SysDbg(X)
#else
#define SysDbg3(X)
#endif
#define SysDbgAssert3(X)		SysDbgAtLevel(3, X)
#define SysDbgMsg3(X)			SysDbgMsgAtLevel(3, X)
#define SysDbgDumpBuf3(X)		SysDbgDumpBufAtLevel(3, X)

#else

//---------------------------------------------------------------------
/** Debug data and code X at compile verbosity >= 3. 
 * @see SysDbg(X)
 */
//---------------------------------------------------------------------
#define SysDbg3(X)

//---------------------------------------------------------------------
/** Debug data and code X at compile and run verbosity >= 3.
 * @see SysDbgAssert(X)
 */
//---------------------------------------------------------------------
#define SysDbgAssert3(X)

//---------------------------------------------------------------------
/** Output message X at compile verbosity >= 3. 
 * @see SysDbgMsg(X)
 */
//---------------------------------------------------------------------
#define SysDbgMsg3(X)

//---------------------------------------------------------------------
/** Output buffer X at compile verbosity >= 3. 
 * @see SysDbgDumpBuf(X)
 */
//---------------------------------------------------------------------
#define SysDbgDumpBuf3(X)

#endif // (SYSTEM_DEBUG_LEVEL >= 3)


// define macros that apply to level 4
#if (SYSTEM_DEBUG_LEVEL >= 4)

#if (SYSTEM_DEBUG_LEVEL_DEFAULT >= 4)
#define SysDbg4(X)				SysDbg(X)
#else
#define SysDbg4(X)
#endif
#define SysDbgAssert4(X)		SysDbgAtLevel(4, X)
#define SysDbgMsg4(X)			SysDbgMsgAtLevel(4, X)
#define SysDbgDumpBuf4(X)		SysDbgDumpBufAtLevel(4, X)

#else

//---------------------------------------------------------------------
/** Debug data and code X at compile verbosity >= 4. 
 * @see SysDbg(X)
 */
//---------------------------------------------------------------------
#define SysDbg4(X)

//---------------------------------------------------------------------
/** Debug data and code X at compile and run verbosity >= 4.
 * @see SysDbgAssert(X)
 */
//---------------------------------------------------------------------
#define SysDbgAssert4(X)

//---------------------------------------------------------------------
/** Output message X at compile verbosity >= 4. 
 * @see SysDbgMsg(X)
 */
//---------------------------------------------------------------------
#define SysDbgMsg4(X)

//---------------------------------------------------------------------
/** Output buffer X at compile verbosity >= 4. 
 * @see SysDbgDumpBuf(X)
 */
//---------------------------------------------------------------------
#define SysDbgDumpBuf4(X)

#endif // (SYSTEM_DEBUG_LEVEL >= 4)



#ifdef __cplusplus
}
#endif

#endif  // __DCSYSDEBUG_H_INCLUDED__

