
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 5 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <popdispatch.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  PopulatorDispatch
//---------------------------------------------------------------------
s32 PopulatorDispatch(_IN u32 popID,
					  _IN u32 reqType,
					  _IN const void *pInBuf, 
					  _IN u32 inBufSize,
					  _OUT void *pOutBuf, 
					  _IN u32 outBufSize,
					  _OUT u32 *pBytesReturned)
{
	s32 status;
	u32 bytesWrittenToOutBuf;


    SysDbgMsg4(("PopulatorDispatch: popID: %u entry\n", 
		popID));

	// default no bytes returned
    *pBytesReturned = 0;
	bytesWrittenToOutBuf = 0;

	// dispatch based on request type
    switch (reqType)
    {
		case SM_DRT_COM_GET_COMPONENT_VERSION:
			// sanity check
			if (outBufSize < sizeof(SMVersionInfo))
			{
				status = SM_STATUS_DATA_OVERRUN;
				break;
			}

			// stack operation so do not need to lock
			((SMVersionInfo *)pOutBuf)->majorVersion = PRD_VER_PRODUCTVERSION;
			((SMVersionInfo *)pOutBuf)->minorVersion = PRD_VER_PRODUCTREVISION;

			// set bytes written
			bytesWrittenToOutBuf = sizeof(SMVersionInfo);

			// done
			status = SM_STATUS_SUCCESS;
		break;

		case SM_DRT_COM_GET_OBJ_LIST_INFO:
			// sanity check
			if (outBufSize < sizeof(SMRspObjListInfo))
			{
				status = SM_STATUS_DATA_OVERRUN;
				break;
			}

			// set inout parameter
			bytesWrittenToOutBuf = outBufSize;

			// perform request
			status = PopDispGetObjListInfo(pOutBuf, 
				&bytesWrittenToOutBuf);
		break;

		case SM_DRT_COM_LIST_CHILDREN_OID:
			// sanity check
			if (inBufSize < sizeof(ObjID))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			if (outBufSize < sizeof(ObjList))
			{
				status = SM_STATUS_DATA_OVERRUN;
				break;
			}

			// set inout parameter
			bytesWrittenToOutBuf = outBufSize;

			// perform request
			status = PopDispListChildrenOID(pInBuf, 
				pOutBuf, 
				&bytesWrittenToOutBuf);
		break;

		case SM_DRT_COM_GET_OBJ_BY_OID:
			// sanity check
			if (inBufSize < sizeof(ObjID))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			if (outBufSize < sizeof(DataObjHeader))
			{
				status = SM_STATUS_DATA_OVERRUN;
				break;
			}

			// set inout parameter
			bytesWrittenToOutBuf = outBufSize;

			// perform request
			status = PopDispGetObjByOID(pInBuf, 
				pOutBuf, 
				&bytesWrittenToOutBuf);
		break;

		case SM_DRT_COM_PASS_THRU_BY_OID:
			// sanity check
			if (inBufSize < sizeof(SMReqHeaderPassThru))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			// set inout parameter
			bytesWrittenToOutBuf = outBufSize;

			// perform request
			status = PopDispPassThru(pInBuf, 
				&inBufSize, 
				pOutBuf, 
				&bytesWrittenToOutBuf);
		break;

		case SM_DRT_POP_LOAD:
			// sanity check
			if (inBufSize < sizeof(SMReqPopLoad))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			// perform request
			status = PopDispLoad(((SMReqPopLoad *)pInBuf)->pSMDPDMDE);
			// return 0 bytes
		break;

		case SM_DRT_POP_UNLOAD:
			// perform request
			status = PopDispUnLoad();
			// return 0 bytes
		break;

		case SM_DRT_POP_START_MONITOR:
			// perform request
			status = PopDispStartMonitor();
			// return 0 bytes
		break;

		case SM_DRT_POP_STOP_MONITOR:
			// perform request
			status = PopDispStopMonitor();
			// return 0 bytes
		break;

		case SM_DRT_POP_REFRESH_OBJ:
			// sanity check
			if (inBufSize < sizeof(DataObjHeader))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			if (outBufSize < sizeof(DataObjHeader))
			{
				status = SM_STATUS_DATA_OVERRUN;
				break;
			}

			// set inout parameter
			bytesWrittenToOutBuf = outBufSize;

			// perform request
			status = PopDispRefreshObj(pInBuf, 
				pOutBuf, 
				&bytesWrittenToOutBuf);
		break;

		case SM_DRT_POP_SET_DATA_EVENT:
			// sanity check
			if (inBufSize < sizeof(DataEventHeader))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			// perform request
			status = PopDispSetDataEvent(pInBuf);
			// return 0 bytes
		break;

		case SM_DRT_POP_SET_OBJ_BY_OID:
			// sanity check
			if (inBufSize < sizeof(SMReqHeaderSet))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			if (outBufSize < sizeof(DataObjHeader))
			{
				status = SM_STATUS_DATA_OVERRUN;
				break;
			}

			// set inout parameter
			bytesWrittenToOutBuf = outBufSize;

			// perform request
			status = PopDispSetObjByOID(pInBuf, 
				&inBufSize, 
				pOutBuf, 
				&bytesWrittenToOutBuf);
		break;

		default:
			status = SM_STATUS_CMD_BAD;
    } // switch (reqType)

	// check if bytesreturned set
	if (status == SM_STATUS_SUCCESS)
	{
		*pBytesReturned = bytesWrittenToOutBuf;
	}

    SysDbgMsg4(("PopulatorDispatch: popID: %u exit\n", 
		popID));

	UNREFERENCED_PARAMETER(popID);

    return status;
} // PopulatorDispatch 

//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------
