//--------------------------------------------------------------------
//
//          DELL COMPUTER CORPORATION PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Computer Corporation and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Computer Corporation. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <popprivatedata.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------

#define POP_PRIVDATA_NUMELEMENTS_INITIALDEFAULT	(192)
#define POP_PRIVDATA_NUMELEMENTS_INCREMENT		(32)
#define POP_PRIVDATA_NUMELEMENTS_MAX			(0x00ffffff)
#define POP_PRIVDATA_ELEMENT_SIZE				(sizeof(PopPrivateDataNode))
#define POP_PRIVDATA_OID_SEED					(0)


// structure of each node in private data table
typedef struct _PopPrivateDataNode
{
	// data object identifier generated for node
	ObjID oid;
	// pointer to the private data associated with oid
	void *pPrivateData;
} PopPrivateDataNode;


// holds information needed by routines in this module
typedef struct _PopPrivateDataTable
{
	// function to notify the user of insert nodes
	PFNNOTIFYPOPPRIVDATA pfnNotifyInsert;
	// function to notify the user of delete nodes
	PFNNOTIFYPOPPRIVDATA pfnNotifyDelete;
	// array of nodes (table) containing populator private data
	PopPrivateDataNode *pPrivateDataNodeTbl;
	// maximum number of elements pPrivateDataNodeTbl can hold
	u32 numElementsMax;
	// current number of elements pPrivateDataNodeTbl holds
	u32 numElementsUsed;
	// TRUE if table is currently sorted
	booln isTableSorted;
	// populator identifier
	u8 popID;
	// alignment for next data element
	u8 reservedAlign[1];
} PopPrivateDataTable;


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

// global data
PopPrivateDataTable *pPPDT = NULL;



//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------



//---------------------------------------------------------------------
// PopPrivateDataCompareNodeByNode
//---------------------------------------------------------------------
int __cdecl PopPrivateDataCompareNodeByNode(const void *pElement1, 
											const void *pElement2)
{
	const PopPrivateDataNode *pPPDN1 = (const PopPrivateDataNode *)pElement1;
	const PopPrivateDataNode *pPPDN2 = (const PopPrivateDataNode *)pElement2;


	// AVR(06/01/02): cannot perform integer 
	// arithmetic to compare the OIDs as u32 
	// is wider than s32 so use expressions

	if (pPPDN1->oid.ObjIDUnion.asu32 == 
		pPPDN2->oid.ObjIDUnion.asu32)
	{
		SysDbgMsg3(("PopPrivateDataCompareNodeByNode: invalid non unique OID: %08X\n", 
			pPPDN1->oid.ObjIDUnion.asu32));
		// equal
		return 0;
	}

	if (pPPDN1->oid.ObjIDUnion.asu32 < 
		pPPDN2->oid.ObjIDUnion.asu32)
	{
		// less than
		return -1;
	}

	// greater than
	return 1;
} // PopPrivateDataCompareNodeByNode


//---------------------------------------------------------------------
// PopPrivateDataCompareOIDByNode
//---------------------------------------------------------------------
int __cdecl PopPrivateDataCompareOIDByNode(const void *pElement1, 
										   const void *pElement2)
{
	const ObjID *pOID1 = (const ObjID *)pElement1;
	const PopPrivateDataNode *pPPDN2 = (const PopPrivateDataNode *)pElement2;


	// AVR(06/01/02): cannot perform integer 
	// arithmetic to compare the OIDs as u32 
	// is wider than s32 so use expressions

	if (pOID1->ObjIDUnion.asu32 == 
		pPPDN2->oid.ObjIDUnion.asu32)
	{
		// equal
		return 0;
	}

	if (pOID1->ObjIDUnion.asu32 < 
		pPPDN2->oid.ObjIDUnion.asu32)
	{
		// less than
		return -1;
	}

	// greater than
	return 1;
} // PopPrivateDataCompareOIDByNode


//---------------------------------------------------------------------
//  PopPrivateDataSortNodeTable
//---------------------------------------------------------------------
void PopPrivateDataSortNodeTable(void)
{
	// any elements present?
	if (pPPDT->numElementsUsed > 1)
	{
		// use qsort to sort the table
		qsort(pPPDT->pPrivateDataNodeTbl, 
			pPPDT->numElementsUsed, 
			POP_PRIVDATA_ELEMENT_SIZE, 
			PopPrivateDataCompareNodeByNode);
	} // if (pPPDT->numElementsUsed > 1)

	// table is sorted
	pPPDT->isTableSorted = TRUE;
} // PopPrivateDataSortNodeTable


//---------------------------------------------------------------------
//  PopPrivateDataAppendLocked
//---------------------------------------------------------------------
s32 PopPrivateDataAppendLocked(_IN const ObjID *pOID, 
							   _IN void *pPrivateData, 
							   _OUT booln *pIsTableSortNeeded)
{
	PopPrivateDataNode *pSaveNodeTbl;
	s32 status;
	u32 tableSizeMax;


	// NOTE: this routine assumes the table is sorted
	// on entry and determines if sort is needed
	// on exit. Additionally it assumes
	// pOID is valid on entry

	// check if we need to grow the table
	if (pPPDT->numElementsUsed == pPPDT->numElementsMax)
	{
		// need to grow table
		// will it fit?
		if ((POP_PRIVDATA_NUMELEMENTS_MAX - POP_PRIVDATA_NUMELEMENTS_INCREMENT) 
			< pPPDT->numElementsMax)
		{
			status = SM_STATUS_DEVICE_FULL;
			_throw error1;
		}

		// safe to increment
		pPPDT->numElementsMax += POP_PRIVDATA_NUMELEMENTS_INCREMENT;

		// compute table size each entry (row)
		// of table contains one node
		tableSizeMax = (pPPDT->numElementsMax * 
			(u32)POP_PRIVDATA_ELEMENT_SIZE);

		// save the old
		pSaveNodeTbl = pPPDT->pPrivateDataNodeTbl;
		// allocate the new table
		pPPDT->pPrivateDataNodeTbl = SMReAllocMem(pPPDT->pPrivateDataNodeTbl, 
			tableSizeMax);

		// failed?
		if (NULL == pPPDT->pPrivateDataNodeTbl)
		{
			SysDbgMsg3(("PopPrivateDataAppendLocked: failed to allocate pPrivateDataNodeTbl\n"));
			// restore table
			pPPDT->pPrivateDataNodeTbl = pSaveNodeTbl;
			pPPDT->numElementsMax -= POP_PRIVDATA_NUMELEMENTS_INCREMENT;
			status = SM_STATUS_OUT_OF_MEMORY;
			_throw error1;
		} // if (NULL == pPPDT->pPrivateDataNodeTbl)
	} // if (pPPDT->numElementsUsed == pPPDT->numElementsMax)

	// any elements to scan?
	if (0 == pPPDT->numElementsUsed)
	{
		// no change to table sort order
		*pIsTableSortNeeded = FALSE;
		// no change to previous table
		// sorted state
	}
	else
	{
		// determine table sort need
		if (pPPDT->pPrivateDataNodeTbl[pPPDT->numElementsUsed-1].oid.ObjIDUnion.asu32
			< pOID->ObjIDUnion.asu32)
		{
			// no change to table sort order
			*pIsTableSortNeeded = FALSE;
			// no change to previous table
			// sorted state
		}
		else
		{
			// append will disrupt sort order
			*pIsTableSortNeeded = TRUE;
			// table is un-sorted
			pPPDT->isTableSorted = FALSE;
		}
	} // if (0 == pPPDT->numElementsUsed)

	// append to table
	pPPDT->pPrivateDataNodeTbl[pPPDT->numElementsUsed].oid = *pOID;
	pPPDT->pPrivateDataNodeTbl[pPPDT->numElementsUsed].pPrivateData = pPrivateData;

	// increment position
	pPPDT->numElementsUsed++;

	// done
	return SM_STATUS_SUCCESS;

error1:
	// no change to table sort order
	*pIsTableSortNeeded = FALSE;
	// no change to previous table
	// sorted state
	// failed
	return status;
} // PopPrivateDataAppendLocked


//---------------------------------------------------------------------
//  PopPrivateDataGetNodePtrByOID
//---------------------------------------------------------------------
PopPrivateDataNode *PopPrivateDataGetNodePtrByOID(_IN const ObjID *pOID)
{

	// sanity check
	if (NULL == pPPDT)
	{
		SysDbgMsg3(("PopPrivateDataGetNodePtrByOID: invalid pPPDT == NULL\n"));
		_throw error1;
	} // if (NULL == pPPDT)

	// valid oid should be specified
	if (NULL == pOID)
	{
		SysDbgMsg3(("PopPrivateDataGetNodePtrByOID: invalid parameter pOID == NULL\n"));
		_throw error1;
	} // if (NULL == pOID)

	if (DATA_OBJ_OID_INVALID == pOID->ObjIDUnion.asu32)
	{
		SysDbgMsg3(("PopPrivateDataGetNodePtrByOID: invalid parameter pOID == DATA_OBJ_OID_INVALID\n"));
		_throw error1;
	} // if (DATA_OBJ_OID_INVALID == pOID->ObjIDUnion.asu32)

	// any elements to scan?
	if (0 == pPPDT->numElementsUsed)
	{
		_throw error1;
	} // if (0 == pPPDT->numElementsUsed)

	// does table need sorting?
	if (FALSE == pPPDT->isTableSorted)
	{
		PopPrivateDataSortNodeTable();
	} // if (FALSE == pPPDT->isTableSorted)

	// locate and return matching node
	return (PopPrivateDataNode *)bsearch((void *)pOID, 
			pPPDT->pPrivateDataNodeTbl, 
			pPPDT->numElementsUsed, 
			POP_PRIVDATA_ELEMENT_SIZE, 
			PopPrivateDataCompareOIDByNode);

error1:
	// failed
	return NULL;
} // PopPrivateDataGetNodePtrByOID


//---------------------------------------------------------------------
//  PopPrivateDataGetAvailableOID
//---------------------------------------------------------------------
ObjID PopPrivateDataGetAvailableOID(void)
{
	ObjID oidRet;
	ObjID oidMatch;
	u32 ii;
	booln isFound;


	// sanity check
	if (NULL == pPPDT)
	{
		SysDbgMsg3(("PopPrivateDataGetAvailableOID: invalid pPPDT == NULL\n"));
		_throw error1;
	} // if (NULL == pPPDT)

	// if the table is unsorted
	// sort it so that the generation is reliable
	if (FALSE == pPPDT->isTableSorted)
	{
		PopPrivateDataSortNodeTable();
	} // if (FALSE == pPPDT->isTableSorted)

	// NOTE: at this stage the table is sorted
	// in ascending order which means the last
	// element in use in table is highest which can
	// be used in order to generate the highest 
	// available (least recently used)

	// non-empty table
	if (0 == pPPDT->numElementsUsed)
	{
		// start at seed
		oidRet.ObjIDUnion.asu32 = POP_PRIVDATA_OID_SEED;
	}
	else
	{
		// get highest in use
		oidRet = pPPDT->pPrivateDataNodeTbl[pPPDT->numElementsUsed-1].oid;

		// is highest in use a public oid?
		if (oidRet.ObjIDUnion.ObjIDCreatorStruct.creatorID 
			!= DATA_CREATOR_ID_PUBLIC)
		{
			// we cannot assume seed since we have
			// generated an oid that is in use

			// drop the creator id
			oidRet.ObjIDUnion.ObjIDCreatorStruct.creatorID = 0;

			// are we maxed out?
			if (POP_PRIVDATA_NUMELEMENTS_MAX == oidRet.ObjIDUnion.asu32)
			{
				// wrap around and search for a slot
				isFound = FALSE;

				// start at seed
				oidRet.ObjIDUnion.asu32 = POP_PRIVDATA_OID_SEED;

				for (ii = 0; (ii < pPPDT->numElementsUsed) && (FALSE == isFound); ii++)
				{
					// skip public oids
					if (pPPDT->pPrivateDataNodeTbl[ii].oid.ObjIDUnion.ObjIDCreatorStruct.creatorID 
						!= DATA_CREATOR_ID_PUBLIC)
					{
						// found non public oid
						oidMatch = pPPDT->pPrivateDataNodeTbl[ii].oid;

						// drop the creator id
						oidMatch.ObjIDUnion.ObjIDCreatorStruct.creatorID = 0;

						// check if we found a slot
						if (oidMatch.ObjIDUnion.asu32 
							== oidRet.ObjIDUnion.asu32)
						{
							// try next slot
							oidRet.ObjIDUnion.asu32++;
						}
						else
						{
							// found unused slot
							isFound = TRUE;
						} // if (oidMatch.ObjIDUnion.asu32
					} // if (pPPDT->pPrivateDataNodeTbl[ii].oid...
				} // for (ii...

				// check if we failed
				if (FALSE == isFound)
				{
					// we are full
					_throw error1;
				} // if (FALSE == isFound)
			}
			else
			{
				// we do not need to wrap around
				// as we are not maxed out
				// increment to next slot
				oidRet.ObjIDUnion.asu32++;
			} // if (POP_PRIVDATA_NUMELEMENTS_MAX == oidRet.ObjIDUnion.asu32)
		}
		else
		{
			// start at seed
			oidRet.ObjIDUnion.asu32 = POP_PRIVDATA_OID_SEED;
		} // if (oidRet.ObjIDUnion.ObjIDCreatorStruct.creatorID
	} // if (pPPDT->numElementsUsed > 0)

	// insert the creator id
	oidRet.ObjIDUnion.ObjIDCreatorStruct.creatorID = pPPDT->popID;

	// done
	return oidRet;

error1:
	// setup failed oid
	oidRet.ObjIDUnion.asu32 = DATA_OBJ_OID_INVALID;
	// failed
	return oidRet;
} // PopPrivateDataGetAvailableOID


//---------------------------------------------------------------------
//  PopPrivateDataInsert
//---------------------------------------------------------------------
s32 PopPrivateDataInsert(_INOUT ObjID *pOID, 
						 _IN void *pPrivateData, 
						 _IN void *pNotifyDataInsert, 
						 _IN booln allocateOID)
{
	PopPrivateDataNode *pPPDN;
	ObjID toid;
	const ObjID *pTOID;
	s32 status;
	booln isTableSortNeeded;

	

	// lock the data here
	PopDataSyncWriteLock();

	// caller specified OID?
	if (FALSE == allocateOID)
	{
		// search the table and find the OID
		pPPDN = PopPrivateDataGetNodePtrByOID(pOID);

		// failed?
		if (NULL != pPPDN)
		{
			status = SM_STATUS_OBJECT_ALREADY_EXISTS;
			_throw error2;
		}

		// validate id
		if (pOID->ObjIDUnion.ObjIDCreatorStruct.creatorID 
			!= DATA_CREATOR_ID_PUBLIC)
		{
			// must belong to this populator
			if (pOID->ObjIDUnion.ObjIDCreatorStruct.creatorID 
				!= pPPDT->popID)
			{
				SysDbgMsg3(("PopPrivateDataInsert: failed popID: %lu match\n", 
					(u32)pOID->ObjIDUnion.ObjIDCreatorStruct.creatorID));
				status = SM_STATUS_OBJECT_BAD;
				_throw error2;
			}
		}

		// save oid
		pTOID = pOID;
	}
	else
	{
		// generate the OID
		toid = PopPrivateDataGetAvailableOID();

		// failed?
		if (DATA_OBJ_OID_INVALID == toid.ObjIDUnion.asu32)
		{
			status = SM_STATUS_DEVICE_FULL;
			_throw error2;
		}

		// setup
		pTOID = &toid;
	} // if (FALSE == allocateOID)

	// append OID at the end grow table if needed
	status = PopPrivateDataAppendLocked(pTOID, 
		pPrivateData, 
		&isTableSortNeeded);

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		_throw error2;
	}

	// copy the OID to the caller if
	// requested
	if (NULL != pOID)
	{
		*pOID = *pTOID;
	} // if (NULL != pOID)

	// notify callback if necessary
	if (NULL != pPPDT->pfnNotifyInsert)
	{
		pPPDT->pfnNotifyInsert(pTOID, 
			pPrivateData, 
			pNotifyDataInsert);
	} // if (NULL != pPPDT->pfnNotifyInsert)

	// unlock the data here
	PopDataSyncWriteUnLock();

	// done
	return SM_STATUS_SUCCESS;	

error2:
	// unlock the data here
	PopDataSyncWriteUnLock();

	return status;
} // PopPrivateDataInsert


//---------------------------------------------------------------------
// PopPrivateDataGet
//---------------------------------------------------------------------
void *PopPrivateDataGet(_IN const ObjID *pOID)
{
	PopPrivateDataNode *pPPDN;
	void *pPrivateData;
	
	
	// lock the data here
	PopDataSyncReadLock();

	// search the table and find the element
	pPPDN = PopPrivateDataGetNodePtrByOID(pOID);

	// failed?
	if (NULL == pPPDN)
	{
		_throw error2;
	} // if (NULL == pPPDN)

	// save pointer
	pPrivateData = pPPDN->pPrivateData;
	
	// unlock data
	PopDataSyncReadUnLock();

	// success
	return pPrivateData;

error2:
	// unlock data
	PopDataSyncReadUnLock();

	// failed
	return NULL;
} // PopPrivateDataGet


//---------------------------------------------------------------------
// PopPrivateDataDelete
//---------------------------------------------------------------------
s32 PopPrivateDataDelete(_IN const ObjID *pOID, 
						 _IN void *pNotifyDataDelete)
{
	s32 status;
	PopPrivateDataNode *pPPDN;
	u32 tableSizeUsed;
	u32 tableSizeMove;
	

	// lock the data here
	PopDataSyncWriteLock();

	// search the table and find the element
	pPPDN = PopPrivateDataGetNodePtrByOID(pOID);

	// failed?
	if (NULL == pPPDN)
	{
		status = SM_STATUS_NO_SUCH_DEVICE;
		_throw error2;
	} // if (NULL == pPPDN)

	// notify callback if necessary
	if (NULL != pPPDT->pfnNotifyDelete)
	{
		pPPDT->pfnNotifyDelete(&pPPDN->oid, 
			pPPDN->pPrivateData, 
			pNotifyDataDelete);
	} // if (NULL != pPPDT->pfnNotifyDelete)

	// was this the last node?
	if (pPPDT->numElementsUsed > 1)
	{
		// compute size of move downward due to delete
		tableSizeUsed = ((pPPDT->numElementsUsed - 1) * 
			(u32)POP_PRIVDATA_ELEMENT_SIZE);

		// compute size to move
		tableSizeMove = (u32)(((u8 *)pPPDT->pPrivateDataNodeTbl + 
			tableSizeUsed) - (u8 *)pPPDN);

		// move down one position
		memmove(pPPDN, 
			((u8 *)pPPDN + POP_PRIVDATA_ELEMENT_SIZE), 
			tableSizeMove);
	} // if (pPPDT->numElementsUsed > 1)

	// decrement the num elements used
	pPPDT->numElementsUsed--;

	// no change to table sort state

	// unlock the data here
	PopDataSyncWriteUnLock();

	return SM_STATUS_SUCCESS;

error2:
	// unlock the data here
	PopDataSyncWriteUnLock();

	return status;
} // PopPrivateDataDelete


//---------------------------------------------------------------------
//  PopPrivateDataAttach
//---------------------------------------------------------------------
s32 PopPrivateDataAttach(_IN u8 popID, 
						 _IN u32 initialNumElements, 
						 _IN PFNNOTIFYPOPPRIVDATA pfnNotifyInsert, 
						 _IN PFNNOTIFYPOPPRIVDATA pfnNotifyDelete)
{
	s32 status;
	u32 tableSizeMax;


	// lock the data here
	PopDataSyncWriteLock();

	// set default table elements if necessary
	if (0 == initialNumElements)
	{
		initialNumElements = POP_PRIVDATA_NUMELEMENTS_INITIALDEFAULT;
	} // if (0 == initialNumElements)

	// sanity check
	if (POP_PRIVDATA_NUMELEMENTS_MAX < initialNumElements)
	{
		SysDbgMsg3(("PopPrivateDataAttach: invalid initialNumElements: %lu\n", 
			initialNumElements));
		status = SM_STATUS_INVALID_PARAMETER;
		_throw error1;
	} // if (POP_PRIVDATA_NUMELEMENTS_MAX < initialNumElements)

	// allocate the table first
	if (NULL == (pPPDT = SMAllocMem(sizeof(PopPrivateDataTable))))
	{
		SysDbgMsg3(("PopPrivateInitPopPrivateDataTable: failed to allocate pPPDT\n"));
		status = SM_STATUS_OUT_OF_MEMORY;
		_throw error1;
	}

	// set the table initial size
	pPPDT->numElementsMax = initialNumElements;
	// no data currently stored
	pPPDT->numElementsUsed	= 0;
	// table is sorted
	pPPDT->isTableSorted = TRUE;

	// save notification callbacks
	pPPDT->pfnNotifyInsert = pfnNotifyInsert;
	pPPDT->pfnNotifyDelete = pfnNotifyDelete;
	
	// compute table size each entry (row)
	// of table contains one node
	tableSizeMax = (initialNumElements * 
		(u32)POP_PRIVDATA_ELEMENT_SIZE);

	// allocate the initial table
	pPPDT->pPrivateDataNodeTbl = SMReAllocMem(NULL, 
		tableSizeMax);

	// failed?
	if (NULL == pPPDT->pPrivateDataNodeTbl)
	{
		SysDbgMsg3(("PopPrivateInitPopPrivateDataTable: failed to allocate pPrivateDataNodeTbl\n"));
		status = SM_STATUS_OUT_OF_MEMORY;
		_throw error2;
	} // if (NULL == pPPDT->pPrivateDataNodeTbl)

	// get populator id
	pPPDT->popID = popID;

	// unlock the data here
	PopDataSyncWriteUnLock();

	// done
	return SM_STATUS_SUCCESS;

error2:
	// free memory
	SMFreeMem(pPPDT);
	// catch GPFs
	pPPDT = NULL;
error1:
	// unlock lock
	PopDataSyncWriteUnLock();
	// failed
	return status;
} // PopPrivateDataAttach


//---------------------------------------------------------------------
//  PopPrivateDataDetach
//---------------------------------------------------------------------
s32 PopPrivateDataDetach(_IN void *pNotifyDataDelete)
{
	u32 elementCount;
	s32 status;
	
	
	// lock the data here
	PopDataSyncWriteLock();

	// sanity check
	if (NULL == pPPDT)
	{
		SysDbgMsg3(("PopPrivateDataDetach: failed pPPDT not allocated\n"));
		status = SM_STATUS_NO_SUCH_DEVICE;
		_throw error1;
	}

	// do we need to notify callback?
	if (NULL != pPPDT->pfnNotifyDelete)
	{
		// first free all the private data elements
		for (elementCount = 0; elementCount < pPPDT->numElementsUsed; elementCount++)
		{
			// is node data present?
			if (NULL != pPPDT->pPrivateDataNodeTbl[elementCount].pPrivateData)
			{
				// notify callback
				pPPDT->pfnNotifyDelete(&pPPDT->pPrivateDataNodeTbl[elementCount].oid, 
					pPPDT->pPrivateDataNodeTbl[elementCount].pPrivateData, 
					pNotifyDataDelete);
			}
		} // for (elementCount = 0
	} // if (NULL != pPPDT->pfnNotifyDelete)

	SysDbgMsg4(("PopPrivateDataDetach: deleted %lu nodes\n", 
		pPPDT->numElementsUsed));
	
	// now free the allocated pointers
	SMFreeMem(pPPDT->pPrivateDataNodeTbl);
	// catch GPFs
	pPPDT->pPrivateDataNodeTbl = NULL;

	// clear the counts
	pPPDT->numElementsUsed = 0;
	pPPDT->numElementsMax = 0;
	pPPDT->isTableSorted = FALSE;
	pPPDT->popID = DATA_CREATOR_ID_PUBLIC;
	
	// now free the table itself
	SMFreeMem(pPPDT);
	// catch GPFs
	pPPDT = NULL;
	
	// unlock the data here
	PopDataSyncWriteUnLock();
	
	// done
	return SM_STATUS_SUCCESS;

error1:
	// unlock lock
	PopDataSyncWriteUnLock();
	// failed
	return status;
} // PopPrivateDataDetach


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

