
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 9 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------

#define MAX_SAMPOP_OBJ_BREADTH						(256)
#define MAX_SAMPOP_OBJ_DEPTH						(16)



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// PopDispGetObjListInfo
// called on reqType: SM_DRT_COM_GET_OBJ_LIST_INFO
//---------------------------------------------------------------------
s32 PopDispGetObjListInfo(_OUT SMRspObjListInfo *pOLI, 
						  _INOUT u32 *pOLISize)
{

	// set sub tree characteristics
	pOLI->maxBreadth = MAX_SAMPOP_OBJ_BREADTH;
	pOLI->maxDepth = MAX_SAMPOP_OBJ_DEPTH;
	pOLI->maxDataObjSize = MAX_DATA_OBJECT_SIZE;

	// set returned size
	*pOLISize = sizeof(SMRspObjListInfo);

	return SM_STATUS_SUCCESS;
} // PopDispGetObjListInfo


//---------------------------------------------------------------------
// PopDispListChildrenOID
// called on reqType: SM_DRT_COM_LIST_CHILDREN_OID
//---------------------------------------------------------------------
s32 PopDispListChildrenOID(_IN const ObjID *pOID, 
						   _OUT ObjList *pOL, 
						   _INOUT u32 *pOLSize)
{
	ObjID oidTemp;
	u32 origOLSize;
	s32 status;
	u8 popID;


	// In this routine all data populator load time
	// OIDs and hence data objects that need to be
	// created are specified

	// In our sample we will create a place holder
	// object termed as storage object as defined
	// in the sample object header
	// we use routines exported by the push interface
	// that assist in the creation of the OID list
	// all other objects are created and refreshed
	// dynamically using the push interface

	// init the OID list
	origOLSize = *pOLSize;
	status = PopDPDMDObjListClear(pOL, 
		&origOLSize, 
		pOLSize);

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		return status;
	}

	// get populator ID
	popID = PopDPDMDGetPopulatorID();

	// get the OID that uniquely identifies the storage object
	// since the popID is used in creation of the OID it is
	// unique accross all data populators however there may
	// be another object of the same type created at the
	// same level by another data populator
	oidTemp = SamPopDOStorageGetOID(popID);

	// create an object under root
	// which represents the storage object
	if (pOID->ObjIDUnion.asu32 == DATA_OBJ_OID_ROOT)
	{
		// add storage object OID to list under ROOT
		status = PopDPDMDObjListAppendOID(&oidTemp, 
			pOL, 
			&origOLSize, 
			pOLSize);
	}
	else
	{
		// at all other levels we dynamically create objects
		// under the storage OID and other OID branches
		status = SM_STATUS_NO_SUCH_OBJECT;
	} // if (pOID->ObjIDUnion.asu32 == DATA_OBJ_OID_ROOT)

	return status;
} // PopListChildrenOID


//---------------------------------------------------------------------
// PopDispGetObjByOID
// called on reqType: SM_DRT_COM_GET_OBJ_BY_OID
//---------------------------------------------------------------------
s32 PopDispGetObjByOID(_IN const ObjID *pOID, 
					   _OUT DataObjHeader *pDOH, 
					   _INOUT u32 *pDOHBufSize)
{
	u8 popID;


	// In this routine the Data Populator returns
	// the Data Object (Header + Body) in the passed
	// buffer to the Data Manager for insertion into
	// the repository. 

	// The OIDs requested by the Data Manager will be
	// that returned in PopListChildrenOID (for load time), or
	// non-cached data objects with a get, set or pass thru
	// request pending against it from from consumers, or 
	// data populators, or
	// OIDs embedded in data events such as
	// DATA_EVENT_INSERT_OID, or 
	// DATA_EVENT_DIRTY_LIST

	// In our sample we choose only to respond to
	// storage object request as all others are
	// created and refresh using push interface

	// get populator ID
	popID = PopDPDMDGetPopulatorID();

	// which OID?
	if (pOID->ObjIDUnion.asu32 == 
		SamPopDOStorageGetOID(popID).ObjIDUnion.asu32)
	{
		// call get obj routine
		return SamPopDOStorageGetObj(pOID, 
			pDOH, 
			pDOHBufSize);
	}

	return SM_STATUS_NO_SUCH_OBJECT;
} // PopDispGetObjByOID 


//---------------------------------------------------------------------
// PopDispRefreshObj
// called on reqType: SM_DRT_POP_REFRESH_OBJ
//---------------------------------------------------------------------
s32 PopDispRefreshObj(_IN const DataObjHeader *pIDOH, 
					  _OUT DataObjHeader *pODOH, 
					  _INOUT u32 *pODOHBufSize)
{

	// In this routine any objects that have indicated
	// to the Data Manager a timer interval based refresh
	// are required to be refreshed and maintained up to date
	// In the pIDOH is the current copy of the Data Object (if cached)
	// or Data Object Header (if non-cached). This servers as a reference
	// while pODOH contains the output buffer large enough to hold the
	// maximum sized data object maintainable by the Data Manager
	// on return pODOH should contain the entire object

	// In our sample we indicate a timer based refresh
	// for the a sample timed refresh object

	// we copy the input object into the output and perform
	// updates in place in our routine
	memmove(pODOH, 
		pIDOH, 
		pIDOH->objSize);

	// refresh the child as needed
	return SamPopStorageObjChildDORefresh(pODOH, 
		pODOHBufSize);
} // PopDispRefreshObj


//---------------------------------------------------------------------
// PopDispPassThru
// called on reqType: SM_DRT_COM_PASS_THRU_BY_OID
//---------------------------------------------------------------------
s32 PopDispPassThru(_IN const SMReqHeaderPassThru *pOPTReq, 
					_IN const u32 *pReqBufSize, 
					_OUT void *pRsp, 
					_INOUT u32 *pRspBufSize)
{

	// In this routine the Data Populator is required to fulfill
	// any pending passthru requests from consumers or data populators

	// In our sample we do support pass thru requests
	// for the storage objects

	// unsupported except for invalid SM_PASSTHRU_TYPE_INVALID
	// used by test tools
	if (pOPTReq->ptCmd == SM_PASSTHRU_TYPE_INVALID)
	{
		return SM_STATUS_CMD_BAD;
	}
	else
	{
		SysDbgMsg3(("PopDispPassThru: pOPTReq->ptCmd: %lu unsupported on OID: %08X\n", 
			pOPTReq->ptCmd, 
			pOPTReq->objID.ObjIDUnion.asu32));
	}

	UNREFERENCED_PARAMETER(pReqBufSize);
	UNREFERENCED_PARAMETER(pRsp);
	UNREFERENCED_PARAMETER(pRspBufSize);

	return SM_STATUS_CMD_NOT_IMPLEMENTED;
} // PopDispPassThru


//---------------------------------------------------------------------
// PopDispSetObjByOID
// called on reqType: SM_DRT_DMG_SET_OBJ_BY_OID
//---------------------------------------------------------------------
s32 PopDispSetObjByOID(_IN const SMReqHeaderSet *pSReq, 
					   _IN const u32 *pSetSize, 
					   _INOUT DataObjHeader *pIODOH, 
					   _INOUT u32 *pIODOHBufSize)
{

	// In this routine the Data Populator is required to fulfill
	// any pending set requests from consumers or data populators

	// In our sample we do support pass thru requests
	// for the storage objects

	// unsupported except for invalid SM_SET_TYPE_INVALID
	// used by test tools
	if (pSReq->setType == SM_SET_TYPE_INVALID)
	{
		return SM_STATUS_CMD_BAD;
	}
	else
	{
		SysDbgMsg3(("PopDispSetObjByOID: pSReq->setType: %lu unsupported on OID: %08X\n", 
			pSReq->setType, 
			pSReq->objID.ObjIDUnion.asu32));
	}

	UNREFERENCED_PARAMETER(pSetSize);
	UNREFERENCED_PARAMETER(pIODOH);
	UNREFERENCED_PARAMETER(pIODOHBufSize);

	return SM_STATUS_CMD_NOT_IMPLEMENTED;
} // PopDispSetObjByOID 


//---------------------------------------------------------------------
// PopDispSetDataEvent
// called on reqType: SM_DRT_POP_SET_DATA_EVENT
//---------------------------------------------------------------------
s32 PopDispSetDataEvent(_IN const DataEventHeader *pDEH)
{


	// This routine is called when and event is
	// passed by the Data Manager to the Data Populator
	// due to an intra pop event or broadcast event

	// currently support debug display of these messages
	SamPopEventListener(pDEH);

	// done
	return SM_STATUS_SUCCESS;
} // PopDispSetDataEvent

