
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 5 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------

// data structure for populator context
typedef struct _SamPopGlobalsData
{
	// denotes if module is exiting set to
	// FALSE on load and TRUE on stop monitor
	booln exitingFlag;
	// alignment for next data element
	u8 reservedAlign[6];

	// NOTE: Data Populator Developers!
	// Place your populator specific data here

	// worker SMThread pointer
	void *pWorkerSMThread;

	// STOR req test SMThread pointer
	void *pReqTestSMThread;

} SamPopGlobalsData;



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

// pointer to populator global data
SamPopGlobalsData *pSPGData = NULL;

void *pSamPopGlobalsLock = NULL;


//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SamPopGlobalsLock
//---------------------------------------------------------------------
void SamPopGlobalsLock(void)
{
	// lock the global mutex lock
	SMMutexLock(pSamPopGlobalsLock, 
		SM_WAIT_MSECS_INFINITE);
} // SamPopGlobalsLock


//---------------------------------------------------------------------
// SamPopGlobalsUnLock
//---------------------------------------------------------------------
void SamPopGlobalsUnLock(void)
{
	// unlock the global mutex lock
	SMMutexUnLock(pSamPopGlobalsLock);
} // SamPopGlobalsUnLock


//---------------------------------------------------------------------
// SamPopGlobalsAlloc
//---------------------------------------------------------------------
s32 SamPopGlobalsAlloc(void)
{
	void *pLock;


	SysDbgMsg4(("SamPopGlobalsAlloc: entry\n"));

	// allocate the global mutex lock
	if ((pSamPopGlobalsLock = SMMutexCreate(NULL)) == NULL)
	{
		SysDbgMsg3(("SamPopGlobalsAlloc: failed to allocate pSamPopGlobalsLock\n"));
		return SM_STATUS_OUT_OF_MEMORY;
	}

	// lock the context
	SamPopGlobalsLock();

	// allocate memory
	if ((pSPGData = (SamPopGlobalsData *)SMAllocMem(sizeof(SamPopGlobalsData))) == NULL)
	{
		SysDbgMsg3(("SamPopGlobalsAlloc: failed to allocate static context memory\n"));
	
		// clear global lock before free
		pLock = pSamPopGlobalsLock;
		// catch GPFs
		pSamPopGlobalsLock = NULL;
		
		// free global mutex lock and unlock context
		SMMutexDestroy(pLock);

		// failed
		return SM_STATUS_OUT_OF_MEMORY;
	} // allocate memory

	// setup important values
	pSPGData->exitingFlag = TRUE;
	pSPGData->pWorkerSMThread = NULL;
	pSPGData->pReqTestSMThread = NULL;

	// unlock the context
	SamPopGlobalsUnLock();

	// done
	return SM_STATUS_SUCCESS;
} // SamPopGlobalsAlloc


//---------------------------------------------------------------------
// SamPopGlobalsFree
//---------------------------------------------------------------------
void SamPopGlobalsFree(void)
{
	void *pLock;


	SysDbgMsg4(("SamPopGlobalsFree: entry\n"));

	// lock the context
	SamPopGlobalsLock();

	// setup important values
	pSPGData->exitingFlag = TRUE;

	// free the memory
	SMFreeMem(pSPGData);
	pSPGData = NULL;

	// clear global lock before free
	pLock = pSamPopGlobalsLock;
	// catch GPFs
	pSamPopGlobalsLock = NULL;
	
	// free global mutex lock
	SMMutexDestroy(pLock);
} // SamPopGlobalsFree


//---------------------------------------------------------------------
// SamPopGlobalsSetExitingFlag
//---------------------------------------------------------------------
booln SamPopGlobalsSetExitingFlag(_IN booln newFlag)
{
	booln oldFlag;


	// sanity check
	if (pSPGData == NULL)
	{
		// module exiting
		return TRUE;
	}

	// lock the context
	SamPopGlobalsLock();

	// save old
	oldFlag = pSPGData->exitingFlag;

	// save new
	pSPGData->exitingFlag = newFlag;

	// unlock the context
	SamPopGlobalsUnLock();

	// done
	return oldFlag;
} // SamPopGlobalsSetExitingFlag


//---------------------------------------------------------------------
// SamPopGlobalsGetExitingFlag
//---------------------------------------------------------------------
booln SamPopGlobalsGetExitingFlag(void)
{
	booln curFlag;


	// sanity check
	if (pSPGData == NULL)
	{
		// module exiting
		return TRUE;
	}

	// lock the context
	SamPopGlobalsLock();

	// save current
	curFlag = pSPGData->exitingFlag;

	// unlock the context
	SamPopGlobalsUnLock();

	// done
	return curFlag;
} // SamPopGlobalsGetExitingFlag


//---------------------------------------------------------------------
// SamPopGlobalsSetWorkerThreadPtr
//---------------------------------------------------------------------
void *SamPopGlobalsSetWorkerThreadPtr(void *pNewWorkerSMThread)
{
	void *pOldWorkerSMThread;


	// sanity check
	if (pSPGData == NULL)
	{
		// module exiting
		return NULL;
	}

	// lock the context
	SamPopGlobalsLock();

	// save old
	pOldWorkerSMThread = pSPGData->pWorkerSMThread;

	// save new
	pSPGData->pWorkerSMThread = pNewWorkerSMThread;

	// unlock the context
	SamPopGlobalsUnLock();

	// done
	return pOldWorkerSMThread;
} // SamPopGlobalsSetWorkerThreadPtr


//---------------------------------------------------------------------
// SamPopGlobalsSetReqTestThreadPtr
//---------------------------------------------------------------------
void *SamPopGlobalsSetReqTestThreadPtr(void *pNewWorkerSMThread)
{
	void *pOldWorkerSMThread;


	// sanity check
	if (pSPGData == NULL)
	{
		// module exiting
		return NULL;
	}

	// lock the context
	SamPopGlobalsLock();

	// save old
	pOldWorkerSMThread = pSPGData->pReqTestSMThread;

	// save new
	pSPGData->pReqTestSMThread = pNewWorkerSMThread;

	// unlock the context
	SamPopGlobalsUnLock();

	// done
	return pOldWorkerSMThread;
} // SamPopGlobalsSetReqTestThreadPtr


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

