
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 29 $
//
//  Last Modified By / On:
//    $Author: Chris_poblete $ / $Date: 8/19/03 11:40a $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Tool Library Module entry code
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    Each tool may include there initializations during module attach
//    and cleanup on module detach by implementing CustomModuleDetach
//    and CustomModuleAttach and setting MODULE_HAS_CUSTOM_INIT in their
//    project/makefile.
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------
#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// SDK include files
//---------------------------------------------------------------------
#include <decmd.h>
#include <dcsysdebug.h>


//--------------------------------------------------------------------
// Local include files
// Function prototypes and other local includes
//--------------------------------------------------------------------
#ifndef _UNICODE
#define _UNICODE // turn on the unicode support for this service
#endif
#ifndef UNICODE
#define UNICODE // turn on the unicode support for this service
#endif

//--------------------------------------------------------------------
// Local Defines and structures
//--------------------------------------------------------------------

#define MOD_STATE_NOT_LOADED					(0)
#define MOD_STATE_LOADED						(1)


// data type allowing read/write modification as atomic entity
// even in presence of asynchronous interrupts in signals
typedef sig_atomic_t atomicrw;


// module exit listener list entry object definition
typedef struct _ModExitListener
{
	PFNDECMODEXITSIGNALLISTENER pfnListener;
	void *pListenerData;
} ModExitListener;


// global context data definition
typedef struct _GlobalContextData
{
	void *pMExitRequestSem; // semaphore to listen for module exit request
	void *pMDetachSem; // semaphore to listen for module detaching
	ModExitListener *pExitSignalListenerList;
	SMECInfo mOrigECI; // original context info
	u32 exitSignalListenerCount;
	u32 mERSCounter; // exit request semaphore request counter
	atomicrw mLoadedAndReadyFlag; // atomic flag used for sync detach operations
	atomicrw mSigHandlerExitFlag; // atomic flag used to detect signal handling occurence
} GlobalContextData;


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

// allocate static sturctures and code
SYS_DBG_DECLARE_GLOBAL_CODEDATA;


// global context data object
// do not use this outside this file
// use an API below if provided
_PRIV GlobalContextData gCD;


//---------------------------------------------------------------------
// Function forward declarations
//---------------------------------------------------------------------

void DECmdModInstallSigHandler(void);


#ifdef CUSTOM_MODULE_ATTACH
extern booln CUSTOM_MODULE_ATTACH(void);
#endif // CUSTOM_MODULE_ATTACH

#ifdef CUSTOM_MODULE_DETACH
extern booln CUSTOM_MODULE_DETACH(void);
#endif // CUSTOM_MODULE_DETACH


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// DECmdModGetOrigECI
//---------------------------------------------------------------------
SMECInfo *DECmdModGetOrigECI(void)
{

	return &gCD.mOrigECI;
} // DECmdModGetOrigECI


//---------------------------------------------------------------------
// DECmdIsModuleExiting
//---------------------------------------------------------------------
booln DECmdIsModuleExiting(void)
{

	// Returns FALSE (module is not loaded and not ready)
	//   if module is not loaded
	//   if module is in the process of loading
	//   if module is in the process of unloading
	return (MOD_STATE_NOT_LOADED == gCD.mLoadedAndReadyFlag);
} // DECmdIsModuleExiting


//---------------------------------------------------------------------
// DECmdModExitListenerPause
//---------------------------------------------------------------------
void DECmdModExitListenerPause(s32 pauseMilliSeconds)
{

	// make sure exit module is loaded and ready
	if (MOD_STATE_LOADED == gCD.mLoadedAndReadyFlag)
	{
		// global data should be valid when loaded and ready flagged

		// increment counter
		gCD.mERSCounter++;
		
		// pause will occur only if the module exit request
		// is in non-signaled state
		SMSemaphoreAcquireWait(gCD.pMExitRequestSem, 
			pauseMilliSeconds);
	}

	return;
} // DECmdModExitListenerPause


//---------------------------------------------------------------------
// DECmdModExitListenerWait
//---------------------------------------------------------------------
void DECmdModExitListenerWait(void)
{

	// make sure exit module is loaded and ready
	if (MOD_STATE_LOADED == gCD.mLoadedAndReadyFlag)
	{
		// global data should be valid when loaded and ready flagged

		// increment counter
		gCD.mERSCounter++;
		
		// block and wait until module exit request recieved
		SMSemaphoreAcquireWait(gCD.pMExitRequestSem, 
			SM_WAIT_MSECS_INFINITE);
	}

	return;
} // DECmdModExitListenerWait


//---------------------------------------------------------------------
// ModExitSignalListenerAdd
//---------------------------------------------------------------------
s32 ModExitSignalListenerAdd(_IN PFNDECMODEXITSIGNALLISTENER pfnListener,
							 _IN void *pListenerData)
{
	ModExitListener *pCursor;
	ModExitListener *pMEL;
	u32 tSize;
	u32 unitSize;
	s32 rstat = SM_STATUS_UNSUCCESSFUL;


	// sanity check
	if (pfnListener != NULL)
	{
		// make sure exit module is loaded and ready
		if (MOD_STATE_LOADED == gCD.mLoadedAndReadyFlag)
		{
			// set unit size
			unitSize = sizeof(ModExitListener);
			// get size required to allocate memory
			tSize = gCD.exitSignalListenerCount + 1;
			tSize *= unitSize;

			// reallocate memory
			pMEL = SMReAllocMem(gCD.pExitSignalListenerList,
				tSize);
			// make sure we have new memory
			if (pMEL == NULL)
			{
				rstat = SM_STATUS_OUT_OF_MEMORY;
			}
			else
			{
				// assign to global data
				gCD.pExitSignalListenerList = pMEL;
				// get pointer to the storage location
				pCursor = gCD.pExitSignalListenerList + 
					gCD.exitSignalListenerCount;
				// copy the data
				pCursor->pfnListener = pfnListener;
				pCursor->pListenerData = pListenerData;
				// update counter
				gCD.exitSignalListenerCount++;
				// good up to this point
				rstat = SM_STATUS_SUCCESS;
			} // if (gCD.pExitSignalListenerList != NULL)
		} // if (MOD_STATE_LOADED == gCD.mLoadedAndReadyFlag)
	} // if (pfnListener != NULL)
	
	return rstat;
} // ModExitSignalListenerAdd


//---------------------------------------------------------------------
// GlobalContextDataDestroy
//---------------------------------------------------------------------
void GlobalContextDataDestroy(void)
{

	// free heap memory
	if (gCD.pMExitRequestSem != NULL)
	{
		SMSemaphoreDestroy(gCD.pMExitRequestSem);
		gCD.pMExitRequestSem = NULL;
	}

	// free heap memory
	if (gCD.pMDetachSem != NULL)
	{
		SMSemaphoreDestroy(gCD.pMDetachSem);
		gCD.pMDetachSem = NULL;
	}

	// free heap memory
	if (gCD.pExitSignalListenerList != NULL)
	{
		SMFreeMem(gCD.pExitSignalListenerList);
		gCD.pExitSignalListenerList = NULL;
	}

	// init the rest
	gCD.mERSCounter = 0;
	gCD.exitSignalListenerCount = 0;
	gCD.mLoadedAndReadyFlag = MOD_STATE_NOT_LOADED;

	return;
} // GlobalContextDataDestroy


//---------------------------------------------------------------------
// GlobalContextDataInit
//---------------------------------------------------------------------
void GlobalContextDataInit(void)
{

	// init data
	gCD.mLoadedAndReadyFlag = MOD_STATE_NOT_LOADED;
	gCD.mERSCounter = 1;
	gCD.mSigHandlerExitFlag = 0;
	gCD.pMDetachSem = NULL;
	gCD.pMExitRequestSem = NULL;
	gCD.pExitSignalListenerList = NULL;
	gCD.exitSignalListenerCount = 0;
	memset(&gCD.mOrigECI,
		0,
		sizeof(SMECInfo));

	return;
} // GlobalContextDataInit


//---------------------------------------------------------------------
// DECmdModSignalHandler
// NOTE: blocked threads using event signal should not 
// use any resource using global lock upon unblocking.
//---------------------------------------------------------------------
void DECmdModSignalHandler(int sigval)
{
	ModExitListener *pListener;
	SMECInfo currECI;
	booln trapAgain = FALSE;


	// make sure we are in the primary context
	SMSetExportContext(&currECI,
		&gCD.mOrigECI);

	SysDbgMsg4(("DECmdModSignalHandler: entry with signal " PRT_S32 "\n", 
		sigval));

	// NOTE:  For Linux, POSIX spec says pthread_mutex_lock and 
	// pthread_mutex_lock cannot be called from a signal handler. 
	// With LinuxThreads the thread stores some information in its 
	// thread structure when a thread is about to block on a mutex. 
	// When you try to use the mutex in the signal handler, the 
	// information gets overwritten. The thread structure is hosed when 
	// you return from the signal handler.

	// check the state of the module exit request sem
	if (MOD_STATE_LOADED == gCD.mLoadedAndReadyFlag)
	{
		// first set module is unloading
		gCD.mLoadedAndReadyFlag = MOD_STATE_NOT_LOADED;

		// browse the exit signal listener list
		pListener = gCD.pExitSignalListenerList;
		while (gCD.exitSignalListenerCount-- > 0)
		{
			SysDbgMsg4(("DECmdModSignalHandler: " \
				"calling mod exit signal listener: " PRT_U32 "\n",
				gCD.exitSignalListenerCount));

			// call them one by one
			pListener->pfnListener(pListener->pListenerData);

			// move to the next
			pListener++;
		}

		SysDbgMsg4(("DECmdModSignalHandler: " \
			"signaling module exit event, count: " PRT_U32 "\n",
			gCD.mERSCounter));
		
		// signal for module exit
		SMSemaphoreRelease(gCD.pMExitRequestSem, 
			gCD.mERSCounter, 
			NULL);

		// reset ers counter
		gCD.mERSCounter = 1;
	}

	// decide whether to trap again
	switch (sigval)
	{
		case SIGINT:
			trapAgain = TRUE;
			break;
			
		case SIGTERM:
			trapAgain = FALSE;
			// In Netware: module detach must be called before exiting this
			// function. Thus this function should be the one releasing
			// global memory
			gCD.mSigHandlerExitFlag = 1;
			// In Netware, threads exit instantanously as soon as the
			// signal handler returns, thus need to wait for module
			// detach to signal to make sure all threads have exited
			// before we continue
			SysDbgMsg4(("DECmdModSignalHandler: waiting for modulemain to finish\n"));
			SMSemaphoreAcquireWait(gCD.pMDetachSem, 
				SM_WAIT_MSECS_INFINITE);
			break;

		default:
			trapAgain = FALSE;
			break;
	} // switch (sigval)
	
	// reinstall if needed
	if (trapAgain == TRUE)
	{
		// reinstall signal handler
		DECmdModInstallSigHandler();

		SysDbgMsg4(("DECmdModSignalHandler: exit with signal retrap\n"));

		// reset context to previous
		SMResetExportContext(&currECI);
	}
	else
	{
		SysDbgMsg4(("DECmdModSignalHandler: exit\n"));

		// reset context to previous
		SMResetExportContext(&currECI);

		// destroy context data before leaving
		GlobalContextDataDestroy();
	}

	return;
} // DECmdModSignalHandler


//---------------------------------------------------------------------
// DECmdModInstallSigHandler
//---------------------------------------------------------------------
void DECmdModInstallSigHandler(void)
{

	// installing the handlers
	// Interrupt from keyboard (ctrl-c)
	signal(SIGINT,
		DECmdModSignalHandler);

	// Termination signal (in Netware, use of "unload")
	signal(SIGTERM,
		DECmdModSignalHandler);

	// Linux NOTE: SIGKILL and SIGSTOP cannot be caught; if used so, the process
	// and its threads are asked to exit without cleanup which could result
	// in undefined behaviour.
} // DECmdModInstallSigHandler


//---------------------------------------------------------------------
// DECmdModuleDetach
//---------------------------------------------------------------------
void DECmdModuleDetach(void)
{

	SysDbgMsg4(("DECmdModuleDetach: entry\n"));
	
	// should be called only once, otherwise abend will occur

#ifdef CUSTOM_MODULE_DETACH
	CUSTOM_MODULE_DETACH();
#endif // CUSTOM_MODULE_DETACH

	if (gCD.mSigHandlerExitFlag != 0)
	{
		// signal module detach is exiting first
		SysDbgMsg4(("DECmdModuleDetach: signaling module detach is called\n"));
		SMSemaphoreRelease(gCD.pMDetachSem, 
			1, 
			NULL);
	}
	else
	{
		// module detach called without any signal caught
		// thus we do the cleanup

		// first set module is unloading
		gCD.mLoadedAndReadyFlag = MOD_STATE_NOT_LOADED;

		SysDbgMsg4(("DECmdModuleDetach: signaling module exit event, " \
			"count: " PRT_U32 "\n",
			gCD.mERSCounter));
		
		// signal for module exit
		SMSemaphoreRelease(gCD.pMExitRequestSem, 
			gCD.mERSCounter, 
			NULL);

		// reset ers counter
		gCD.mERSCounter = 1;

		// destroy context data before leaving
		GlobalContextDataDestroy();
	}
	
	SysDbgMsg4(("DECmdModuleDetach: exit\n"));
} // DECmdModuleDetach


//---------------------------------------------------------------------
// DECmdModuleAttach
//---------------------------------------------------------------------
booln DECmdModuleAttach(void)
{

	SysDbgMsg4(("DECmdModuleAttach: entry\n"));

	// should be called only once, 
	// otherwise memory leak or deadlock will occur

	// init the globals
	GlobalContextDataInit();

	// init global context info
	SMGetExportContext(&gCD.mOrigECI);

	// first install the default signal handlers
	DECmdModInstallSigHandler();

	// sort the Command Dispatch Table first
	if (DECmdSortCmdDispatchTable() != SM_STATUS_SUCCESS)
	{
		_throw error1;
	}

	// create exiting event 
	// counting semaphore init to 0
	gCD.pMExitRequestSem = SMSemaphoreCreate(NULL, 0);
	if (gCD.pMExitRequestSem == NULL)
	{
		_throw error1;
	}

	// create exiting event 
	// counting semaphore init to 0
	gCD.pMDetachSem = SMSemaphoreCreate(NULL, 0);
	if (gCD.pMDetachSem == NULL)
	{
		_throw error2;
	}

#ifdef CUSTOM_MODULE_ATTACH
	if (CUSTOM_MODULE_ATTACH() == FALSE)
	{
		_throw error3;
	}
#endif // CUSTOM_MODULE_ATTACH

	// now safe to open module gate
	gCD.mLoadedAndReadyFlag = MOD_STATE_LOADED;

	SysDbgMsg4(("DECmdModuleAttach: exit\n"));
	return TRUE;

#ifdef CUSTOM_MODULE_ATTACH
 error3:
	// destroy the detach signal
	SMSemaphoreDestroy(gCD.pMDetachSem);
	// prevent GPFs
	gCD.pMDetachSem = NULL;
#endif // CUSTOM_MODULE_ATTACH
 error2:
	// destroy the exit signal
	SMSemaphoreDestroy(gCD.pMExitRequestSem);
	// prevent GPFs
	gCD.pMExitRequestSem = NULL;
 error1:
	// error
	SysDbgMsg4(("DECmdModuleAttach: exit with error\n"));
	return FALSE;
} // DECmdModuleAttach


//---------------------------------------------------------------------
// CmdExecute - Called by ModuleMain
// This is to provide a higher level of control for the scripting code
//---------------------------------------------------------------------
static s32 CmdExecute(_IN s32 argc, 
					  _IN astring** argv, 
					  _IN DECmdControl* pCmdOp)
{
	astring** ppNVP = NULL;
	u32 countNVP;
	u32 ii;
	s32 rstat;


	// init the control data
	rstat = DECmdControlAttach(argc, 
		argv, 
		pCmdOp, 
		&countNVP);

	if (rstat != SM_STATUS_SUCCESS)
	{
		// print the help if not successful
		DECmdPrintHelp(pCmdOp, 
			argc, 
			argv);

		if (rstat == SM_STATUS_UNSUCCESSFUL)
		{
			// command control is not initialized
			_throw error1;
		}

		// free command control
		_throw error2;
	}

	// process send command only if command name is present
	// or command help is requested
	if ((pCmdOp->cmdName[0] != '\0') ||
		(SMMBitMapIsOn(pCmdOp->optionsMap, DECMDCTRL_OPT_HELP_CMD)))
	{
		// calculate number of expected NVP
		countNVP = (argc - countNVP);
		
		// allocate memory to hold a list of nonNVP strings from argv
		ii = (countNVP * sizeof(astring *));
		if ((ppNVP = (astring **)SMAllocMem(ii)) == NULL)
		{ 
			// CAP:02/10/2003 - In Netware a malloc of 0 returns NULL
			// Thus if there is no command parameter to copy just continue
			// and make sure the count is 0
			countNVP = 0;
		}
		else
		{
			// populate NVP list, discount program name and command argument
			for (ii = 2; ii < (countNVP + 2); ii++)
			{
				ppNVP[ii - 2] = argv[ii];
			}
		}

		// call the command
		rstat = DECmdSendCmd(pCmdOp, 
			countNVP, 
			ppNVP);
		if (rstat == SM_STATUS_INVALID_PARAMETER)
		{
			// set the param help flag and resend
			SMMBitMapSetOn(pCmdOp->optionsMap, DECMDCTRL_OPT_HELP_PARAM);
			
			// print the header first
			DECmdPrintHelpHeaderUsage(pCmdOp,
				argc,
				argv);
			
			// send it again with help request
			DECmdSendCmd(pCmdOp, 
				countNVP, 
				ppNVP);
		}
		else if (rstat != SM_STATUS_SUCCESS)
		{
			// dump error messages if any
			DECmdPrintParamErrMssg(pCmdOp);
		}
	}

	// free NVP list
	SMFreeMem(ppNVP);
	ppNVP = NULL;

 error2:
	// detach control data
	DECmdControlDetach(pCmdOp);
	pCmdOp = NULL;
 error1:
	// exit
	return rstat;
} // CmdExecute


//---------------------------------------------------------------------
// ModuleMain - Called by genmain.c
//---------------------------------------------------------------------
s32 ModuleMain(_IN s32 argc, 
			   _IN astring **argv)
{
	FILE* pScriptFile = NULL;
	astring* pScriptLine = NULL;
	astring* pBuffer = NULL;
	astring* argv0 = argv[0];
	astring** scriptArgv = NULL;
	DECmdControl cmdOp;
	s32 ii;
	s32 rstat = SM_STATUS_UNSUCCESSFUL;
	s32 scriptArgc = 0;
	s32 scriptErrorCount = 0;
	s32 scriptMaxErrors = 0;
	booln IsScript = FALSE;
	booln ErrorCountExceeded = FALSE;


	// initialize the debug trace subsystem
    SysDbgInit((SYSTEM_DEBUG_SCREEN | SYSTEM_DEBUG_FILE), 
		DEFAULT_SYSDBG_FILENAME);

	if (DECmdModuleAttach() != TRUE)
	{
		// just exit
		_throw error1;
	}

	// init cmd control
	memset(&cmdOp, 
		0, 
		sizeof(DECmdControl));

	// is this a script command. If so reload the arguments from the script
	for (ii=1; ii<argc-1; ii++)
	{
		if (0 == stricmp(argv[ii], "-script")) 
		{ 
			// the very next param must be the scriptfile
			pScriptFile = fopen(argv[ii+1], "r");
			if (!pScriptFile)
			{
				// unable to open scriptfile
				DECmdPrintParamErrMssg(&cmdOp);
				_throw error2;
			}
			else
			{
				// scriptfile is open
				IsScript = TRUE;
			}

			if (argc > 4)
			{
				if (0 == stricmp(argv[ii+2], "-maxerrors"))
				{
					scriptMaxErrors = atoi(argv[ii+3]);
				}
			}
			ii = argc+1;
		} // if (0 == stricmp(argv[ii], "-script")) 
	} // for (ii=1; ii<argc-1; ii++)

	if (IsScript)
	{
		pBuffer = SMAllocMem(DECMD_MAX_PARAM_SIZE+1);

		while (!ErrorCountExceeded && feof(pScriptFile) == 0)
		{
			pScriptLine = pBuffer;
			strcpy(pScriptLine, "");
			if(fgets(pScriptLine, DECMD_MAX_PARAM_SIZE, pScriptFile))
			{
				// strip any leading whitespace 
				while(*pScriptLine == ' ' || *pScriptLine == '\t')
				{
					pScriptLine++;
				}

				pScriptLine[strlen(pScriptLine)-1] = '\0';	// get rid of trailing \n from fgets()

				if((strlen(pScriptLine) > 0) &&  (0 != strcspn(pScriptLine, ";#")))
				{

					rstat = CmdScriptParseLine(argv0, 
							pScriptLine, 
							&scriptArgc, 
							&scriptArgv);
					if (rstat != SM_STATUS_SUCCESS)
					{
						DECmdPrintHelp(&cmdOp, 
							argc, 
							argv);
						_throw error2;
					}

					// replace argc and argv with the new values from the script
					SysDbgMsg4(("pScriptLine: %s\n", pScriptLine));

					rstat = CmdExecute(scriptArgc, 
						scriptArgv, 
						&cmdOp);

					if (rstat != SM_STATUS_SUCCESS)
					{
						scriptErrorCount++;
					}

					if (scriptMaxErrors && (scriptErrorCount >= scriptMaxErrors))
					{
						ErrorCountExceeded = TRUE;
					}

					SMFreeMem(&scriptArgv[0]);
					scriptArgv = 0;
				}
			}
		} // while (!ErrorCountExceeded && feof(pScriptFile) == 0)

		// close the script file
		fclose(pScriptFile);
		pScriptFile = NULL;

		// free local mem
		SMFreeMem(pScriptLine);
		pScriptLine = NULL;
	}
	else
	{
		rstat = CmdExecute(argc, 
			argv, 
			&cmdOp);
	} // if (IsScript)

 error2:
	// module detach
	DECmdModuleDetach();
 error1:
	// module exit
	return rstat;
} // ModuleMain


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

