
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 8 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:14p $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    MIB Plug-in Implementer VarBind support functions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core include files
//-----------------------------------------------------------------------------

#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcsysdebug.h>
#include <dcsmstatus.h>
#include <dcsupapi.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP include files
//-----------------------------------------------------------------------------

#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP MPI include files
//-----------------------------------------------------------------------------

#include <mpivarbind.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// MPIVarBindValidateNameScalar - validate name for scalar variable and
// return pointer to AttrInfo struct for attribute if name is valid
//=============================================================================
s32 MPIVarBindValidateNameScalar(_IN const SMSnmpVarBind * pVB,
								 _IN ObjInfo * pObjInfo,
								 _OUT AttrInfo ** ppAttrInfo)
{
	u32 numIdsRequired;
	u32 attrID;


	// determine number of IDs required for scalar OID
	numIdsRequired = pObjInfo->oib_id_ln + 2;

	// validate number of IDs in OID
	if (pVB->name.numIds != numIdsRequired)
	{
		SysDbgMsg3(("MPIVarBindValidateNameScalar: ERROR: invalid numIds: numIds received: %u numIds required: %u\n",
			pVB->name.numIds, numIdsRequired));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate null terminator for scalar OID
	if (pVB->name.ids[pVB->name.numIds - 1] != 0)
	{
		SysDbgMsg3(("MPIVarBindValidateNameScalar: ERROR: invalid terminator for scalar OID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate attribute ID
	attrID = pVB->name.ids[pObjInfo->oib_id_ln];
	if ((attrID == 0) || (attrID > pObjInfo->oib_attr_mx))
	{
		SysDbgMsg3(("MPIVarBindValidateNameScalar: ERROR: invalid attribute: %u\n",
			attrID));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return pointer to AttrInfo for attribute
	*ppAttrInfo = &pObjInfo->oib_attr_pt[attrID];

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindValidateNameScalar


//=============================================================================
// MPIVarBindValidateNameTable1Idx - validate name for table variable with
// one index and return index and pointer to AttrInfo for attribute
// if name is valid
//=============================================================================
s32 MPIVarBindValidateNameTable1Idx(_IN const SMSnmpVarBind * pVB,
									_IN ObjInfo * pObjInfo,
									_OUT AttrInfo ** ppAttrInfo,
									_OUT u32 * pIndex)
{
	u32 numIdsRequired;
	u32 attrID;


	// determine number of IDs required for table variable OID with 1 index
	numIdsRequired = pObjInfo->oib_id_ln + 2;

	// validate number of IDs in OID
	if (pVB->name.numIds != numIdsRequired)
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable1Idx: ERROR: invalid numIds: numIds received: %u numIds required: %u\n",
			pVB->name.numIds, numIdsRequired));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate table entry ID
	if (pVB->name.ids[pObjInfo->oib_id_ln - 1] !=
		pObjInfo->oib_id_pt[pObjInfo->oib_id_ln - 1])
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable1Idx: ERROR: invalid table entry ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate attribute ID
	attrID = pVB->name.ids[pObjInfo->oib_id_ln];
	if ((attrID == 0) || (attrID > pObjInfo->oib_attr_mx))
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable1Idx: ERROR: invalid attribute: %u\n",
			attrID));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return pointer to AttrInfo for attribute
	*ppAttrInfo = &pObjInfo->oib_attr_pt[attrID];

	// return index
	*pIndex = pVB->name.ids[pVB->name.numIds - 1];
	if (*pIndex == 0)
	{
		SysDbgMsg4(("MPIVarBindValidateNameTable1Idx: ERROR: index is zero\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindValidateNameTable1Idx


//=============================================================================
// MPIVarBindValidateNameTable2Idx - validate name for table variable with
// two indexes and return indexes and pointer to AttrInfo for attribute
// if name is valid
//=============================================================================
s32 MPIVarBindValidateNameTable2Idx(_IN const SMSnmpVarBind * pVB,
									_IN ObjInfo * pObjInfo,
									_OUT AttrInfo ** ppAttrInfo,
									_OUT u32 * pIndex1,
									_OUT u32 * pIndex2)
{
	u32 numIdsRequired;
	u32 attrID;


	// determine number of IDs required for table variable OID with 2 indexes
	numIdsRequired = pObjInfo->oib_id_ln + 3;

	// validate number of IDs in OID
	if (pVB->name.numIds != numIdsRequired)
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable2Idx: ERROR: invalid numIds: numIds received: %u numIds required: %u\n",
			pVB->name.numIds, numIdsRequired));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate table entry ID
	if (pVB->name.ids[pObjInfo->oib_id_ln - 1] !=
		pObjInfo->oib_id_pt[pObjInfo->oib_id_ln - 1])
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable2Idx: ERROR: invalid table entry ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate attribute ID
	attrID = pVB->name.ids[pObjInfo->oib_id_ln];
	if ((attrID == 0) || (attrID > pObjInfo->oib_attr_mx))
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable2Idx: ERROR: invalid attribute: %u\n",
			attrID));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return pointer to AttrInfo for attribute
	*ppAttrInfo = &pObjInfo->oib_attr_pt[attrID];

	// return index 1
	*pIndex1 = pVB->name.ids[pVB->name.numIds - 2];
	if (*pIndex1 == 0)
	{
		SysDbgMsg4(("MPIVarBindValidateNameTable2Idx: ERROR: index 1 is zero\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return index 2
	*pIndex2 = pVB->name.ids[pVB->name.numIds - 1];
	if (*pIndex2 == 0)
	{
		SysDbgMsg4(("MPIVarBindValidateNameTable2Idx: ERROR: index 2 is zero\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindValidateNameTable2Idx


//=============================================================================
// MPIVarBindValidateNameTable3Idx - validate name for table variable with
// three indexes and return indexes and pointer to AttrInfo for attribute
// if name is valid
//=============================================================================
s32 MPIVarBindValidateNameTable3Idx(_IN const SMSnmpVarBind * pVB,
									_IN ObjInfo * pObjInfo,
									_OUT AttrInfo ** ppAttrInfo,
									_OUT u32 * pIndex1,
									_OUT u32 * pIndex2,
									_OUT u32 * pIndex3)
{
	u32 numIdsRequired;
	u32 attrID;


	// determine number of IDs required for table variable OID with 3 indexes
	numIdsRequired = pObjInfo->oib_id_ln + 4;

	// validate number of IDs in OID
	if (pVB->name.numIds != numIdsRequired)
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable3Idx: ERROR: invalid numIds: numIds received: %u numIds required: %u\n",
			pVB->name.numIds, numIdsRequired));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate table entry ID
	if (pVB->name.ids[pObjInfo->oib_id_ln - 1] !=
		pObjInfo->oib_id_pt[pObjInfo->oib_id_ln - 1])
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable3Idx: ERROR: invalid table entry ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// validate attribute ID
	attrID = pVB->name.ids[pObjInfo->oib_id_ln];
	if ((attrID == 0) || (attrID > pObjInfo->oib_attr_mx))
	{
		SysDbgMsg3(("MPIVarBindValidateNameTable3Idx: ERROR: invalid attribute: %u\n",
			attrID));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return pointer to AttrInfo for attribute
	*ppAttrInfo = &pObjInfo->oib_attr_pt[attrID];

	// return index 1
	*pIndex1 = pVB->name.ids[pVB->name.numIds - 3];
	if (*pIndex1 == 0)
	{
		SysDbgMsg4(("MPIVarBindValidateNameTable3Idx: ERROR: index 1 is zero\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return index 2
	*pIndex2 = pVB->name.ids[pVB->name.numIds - 2];
	if (*pIndex2 == 0)
	{
		SysDbgMsg4(("MPIVarBindValidateNameTable3Idx: ERROR: index 2 is zero\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return index 3
	*pIndex3 = pVB->name.ids[pVB->name.numIds - 1];
	if (*pIndex3 == 0)
	{
		SysDbgMsg4(("MPIVarBindValidateNameTable3Idx: ERROR: index 3 is zero\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindValidateNameTable3Idx


//=============================================================================
// MPIVarBindValidateSetCommon - perform common validation for Set request
//=============================================================================
s32 MPIVarBindValidateSetCommon(_IN const SMSnmpVarBind * pVB,
								_IN AttrInfo * pAttrInfo)
{

	// can this attribute be set?
	if ((pAttrInfo->aib_flags & AIF_WRITE) == 0)
	{
		SysDbgMsg3(("MPIVarBindValidateSetCommon: ERROR: attribute is read-only\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// is the new value type correct?
	if (pVB->value.type != (u32)pAttrInfo->aib_asn_type)
	{
		SysDbgMsg3(("MPIVarBindValidateSetCommon: ERROR: incorrect type: type received: %02Xh type required: %02Xh\n",
			pVB->value.type, (u32)pAttrInfo->aib_asn_type));
		return SM_SNMP_STATUS_BAD_VALUE;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindValidateSetCommon


//=============================================================================
// MPIVarBindGenerateNameScalar - generate name for scalar variable
// (should only be used to generate scalar names for trap variables)
//=============================================================================
s32 MPIVarBindGenerateNameScalar(_OUT SMSnmpVarBind * pVB,
								 _IN ObjInfo * pObjInfo,
								 _IN AttrInfo * pAttrInfo)
{

	// determine number of IDs for name
	// (numIds = base IDs + attr ID + null terminator)
	pVB->name.numIds = pObjInfo->oib_id_ln + 2;

	// allocate memory for IDs
	pVB->name.ids = SMAllocMem(SM_SNMP_OID_NUMIDS2SIZE(pVB->name.numIds));
	if (pVB->name.ids == NULL)
	{
		SysDbgMsg3(("MPIVarBindGenerateNameScalar: ERROR: no memory\n"));
		pVB->name.numIds = 0;
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// copy base IDs
	memcpy(pVB->name.ids,
		   pObjInfo->oib_id_pt,
		   SM_SNMP_OID_NUMIDS2SIZE(pObjInfo->oib_id_ln));

	// add attribute ID and null terminator
	pVB->name.ids[pObjInfo->oib_id_ln] = pAttrInfo->aib_id;
	pVB->name.ids[pObjInfo->oib_id_ln + 1] = 0;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindGenerateNameScalar


//=============================================================================
// MPIVarBindSetValueInt32 - set 32-bit integer value into VarBind
//=============================================================================
s32 MPIVarBindSetValueInt32(_INOUT SMSnmpVarBind * pVB,
							_IN AttrInfo * pAttrInfo,
							_IN u32 val32)
{

	// set the value type
	pVB->value.type = pAttrInfo->aib_asn_type;

	// put the value in the VarBind
	pVB->value.val32 = val32;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindSetValueInt32


//=============================================================================
// MPIVarBindSetValueOctStr - set octet string value into VarBind
//=============================================================================
s32 MPIVarBindSetValueOctStr(_INOUT SMSnmpVarBind * pVB,
							 _IN AttrInfo * pAttrInfo,
							 _IN void * pOctetString,
							 _IN u32 size,
							 _IN booln allocateBuffer)
{

	// allocate buffer for value?
	// (should only be TRUE for traps)
	if ((size != 0) && (allocateBuffer == TRUE))
	{
		// yes, allocate memory for value
		if ((pVB->value.valptr = SMAllocMem(size)) == NULL)
		{
			SysDbgMsg3(("MPIVarBindSetValueOctStr: ERROR: no memory\n"));
			return SM_SNMP_STATUS_GEN_ERR;
		}
		pVB->value.val32 = size;
	}

	// will the value fit in the VarBind value buffer?
	if ((size != 0) && (size > pVB->value.val32))
	{
		// no, return the buffer size needed
		SysDbgMsg3(("MPIVarBindSetValueOctStr: buffer size received: %u, buffer size needed: %u\n",
			pVB->value.val32, size));
		pVB->value.val32 = size;
		return SM_SNMP_STATUS_TOO_BIG;
	}

	// make sure we have data if size not zero
	if ((size != 0) && (pOctetString == NULL))
	{
		SysDbgMsg3(("MPIVarBindSetValueOctStr: ERROR: no data\n"));
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// set the value type
	pVB->value.type = pAttrInfo->aib_asn_type;

	// put the value in the VarBind value buffer
	if (pOctetString != NULL)
	{
		memcpy(pVB->value.valptr, pOctetString, size);
	}

	// val32 = number of octets
	pVB->value.val32 = size;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindSetValueOctStr


//=============================================================================
// MPIVarBindSetValueOid - set object identifier value into VarBind
//=============================================================================
s32 MPIVarBindSetValueOid(_INOUT SMSnmpVarBind * pVB,
						  _IN AttrInfo * pAttrInfo,
						  _IN SMSnmpOid * pOid,
						  _IN booln allocateBuffer)
{
	u32 size;


	// sanity check
	if ((pOid == NULL) || (pOid->numIds == 0) || (pOid->ids == NULL))
	{
		SysDbgMsg3(("MPIVarBindSetValueOid: ERROR: invalid OID\n"));
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// determine buffer size needed for value
	size = SM_SNMP_OID_NUMIDS2SIZE(pOid->numIds);

	// allocate buffer for value?
	// (should only be TRUE for traps)
	if (allocateBuffer == TRUE)
	{
		// yes, allocate memory for value
		if ((pVB->value.valptr = SMAllocMem(size)) == NULL)
		{
			SysDbgMsg3(("MPIVarBindSetValueOid: ERROR: no memory\n"));
			return SM_SNMP_STATUS_GEN_ERR;
		}
		pVB->value.val32 = size;
	}

	// will the value fit in the VarBind value buffer?
	if (size > pVB->value.val32)
	{
		// no, return the buffer size needed
		SysDbgMsg3(("MPIVarBindSetValueOid: buffer size received: %u, buffer size needed: %u\n",
			pVB->value.val32, size));
		pVB->value.val32 = size;
		return SM_SNMP_STATUS_TOO_BIG;
	}

	// set the value type
	pVB->value.type = pAttrInfo->aib_asn_type;

	// put the value in the VarBind value buffer
	memcpy(pVB->value.valptr, pOid->ids, size);

	// val32 = number of IDs
	pVB->value.val32 = pOid->numIds;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindSetValueOid


//=============================================================================
// MPIVarBindSetValueUCS2ToUTF8Str - convert UCS2 string to UTF8 and
// set UTF8 string value into VarBind
//=============================================================================
s32 MPIVarBindSetValueUCS2ToUTF8Str(_INOUT SMSnmpVarBind * pVB,
									_IN AttrInfo * pAttrInfo,
									_IN ustring * pUStr)
{
	u32 size;
	s32 status;


	// convert UCS2 string value directly into the VarBind value buffer
	size = pVB->value.val32;
	status = SMUCS2StrToUTF8Str(pVB->value.valptr, &size, pUStr);
	if (status != SM_STATUS_SUCCESS)
	{
		if (status == SM_STATUS_DATA_OVERRUN)
		{
			// value buffer not big enough; return the buffer size needed
			size = pVB->value.val32 * 2;
			SysDbgMsg3(("MPIVarBindSetValueUCS2ToUTF8Str: buffer size received: %u, buffer size needed: %u\n",
				pVB->value.val32, size));
			pVB->value.val32 = size;
			return SM_SNMP_STATUS_TOO_BIG;
		}
		else
		{
			SysDbgMsg3(("MPIVarBindSetValueUCS2ToUTF8Str: SMUCS2StrToUTF8Str failed: status: %d\n",
				status));
			return SM_SNMP_STATUS_GEN_ERR;
		}
	}

	// set the value size (excluding NULL terminator) to number of octets
	pVB->value.val32 = size - 1;

	// set the value type
	pVB->value.type = pAttrInfo->aib_asn_type;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindSetValueUCS2ToUTF8Str


//=============================================================================
// MPIVarBindClean - clean a VarBind by freeing any memory allocated
// for the name and value buffers and by resetting the name and value info
// (should only be used to clean trap variables)
//=============================================================================
s32 MPIVarBindClean(_INOUT SMSnmpVarBind * pVB)
{

	// clean the name
	if (pVB->name.ids != NULL)
	{
		// free the name IDs buffer
		SMFreeMem(pVB->name.ids);
		pVB->name.ids = NULL;
	}
	pVB->name.numIds = 0;

	// clean the value
	switch (pVB->value.type)
	{
		case SM_SNMP_TYPE_OCTET_STRING:
		case SM_SNMP_TYPE_OBJ_ID:
		case SM_SNMP_TYPE_IP_ADDRESS:
		case SM_SNMP_TYPE_OPAQUE:
			if (pVB->value.valptr != NULL)
			{
				// free the value buffer
				SMFreeMem(pVB->value.valptr);
			}
			break;
	}
	pVB->value.type = 0;
	pVB->value.valptr = NULL;
	pVB->value.val32 = 0;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindClean


//=============================================================================
// MPIVarBindComputeValueChecksum - compute checksum for VarBind value
// as required for OpenManage Set security
//=============================================================================
s32 MPIVarBindComputeValueChecksum(_IN const SMSnmpVarBind * pVB,
								   _OUT u32 * pChecksum)
{
	u8 * pOctet;
	u32 checksum;
	u32 size;
	u32 ii;


	// calculate checksum on value in VarBind
	switch (pVB->value.type)
	{
		case SM_SNMP_TYPE_INTEGER:
		case SM_SNMP_TYPE_COUNTER:
		case SM_SNMP_TYPE_GAUGE:
		case SM_SNMP_TYPE_TIMETICKS:
			// checksum = value
			checksum = pVB->value.val32;
			break;

		case SM_SNMP_TYPE_OCTET_STRING:
		case SM_SNMP_TYPE_IP_ADDRESS:
		case SM_SNMP_TYPE_OPAQUE:
			// checksum = sum of octet values
			checksum = 0;
			size = pVB->value.val32;
			pOctet = (u8 *)pVB->value.valptr;
			for (ii = 0; ii < size; ii++)
			{
				checksum += pOctet[ii];
			}
			break;

		case SM_SNMP_TYPE_OBJ_ID:
			// checksum = sum of octet values
			checksum = 0;
			size = SM_SNMP_OID_NUMIDS2SIZE(pVB->value.val32);
			pOctet = (u8 *)pVB->value.valptr;
			for (ii = 0; ii < size; ii++)
			{
				checksum += pOctet[ii];
			}
			break;

		default:
			SysDbgMsg3(("MPIVarBindComputeValueChecksum: ERROR: unsupported SNMP data type\n"));
			return SM_SNMP_STATUS_BAD_VALUE;
	}

	// zero is changed to one
	if (checksum == 0)
	{
		checksum = 1;
	}

	// return checksum
	*pChecksum = checksum;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIVarBindComputeValueChecksum


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

