
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    SNMP OID support functions that use secure string functions
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core include files
//-----------------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcsysdebugsecure.h>
#include <dcsupapi.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP include files
//-----------------------------------------------------------------------------
#include <dcsmsnmp.h>
#include <snmpoidsecure.h>


//-----------------------------------------------------------------------------
// Local Definitions
//-----------------------------------------------------------------------------
#define MAX_SNMP_OID_ID_DIGITS			10


//-----------------------------------------------------------------------------
// Global Data Area
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// SnmpOidAlloc - allocate SNMP OID struct
//=============================================================================
SMSnmpOid * SnmpOidAlloc(void)
{
	SMSnmpOid * pOid;


	// allocate memory for SMSnmpOid struct
	pOid = SMAllocMem(sizeof(SMSnmpOid));
SysDbg3(
	if (pOid == NULL)
	{
		SysDbgMsg3(("SnmpOidAlloc: ERROR: no memory\n"));
	}
) // SysDbg3

	return pOid;
} // SnmpOidAlloc


//=============================================================================
// SnmpOidCopy - copy SNMP OID (by allocating memory for and copying IDs)
//=============================================================================
s32 SnmpOidCopy(_IN const SMSnmpOid * pSrcOid,
				_OUT SMSnmpOid * pDstOid)
{
	u32 size;


	// determine memory size needed for IDs
	size = SM_SNMP_OID_NUMIDS2SIZE(pSrcOid->numIds);

	// allocate memory for IDs
	if ((pDstOid->ids = SMAllocMem(size)) == NULL)
	{
		SysDbgMsg3(("SnmpOidCopy: ERROR: no memory for IDs\n"));
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// copy OID
	memcpy(pDstOid->ids, pSrcOid->ids, size);
	pDstOid->numIds = pSrcOid->numIds;

	return SM_SNMP_STATUS_NO_ERROR;
} // SnmpOidCopy


//=============================================================================
// SnmpOidAllocAndCopy - allocate and copy SNMP OID
//=============================================================================
SMSnmpOid * SnmpOidAllocAndCopy(_IN const SMSnmpOid * pSrcOid)
{
	SMSnmpOid * pNewOid;


	// allocate struct for new SNMP OID
	if ((pNewOid = SnmpOidAlloc()) == NULL)
	{
		return NULL;
	}

	// copy OID
	if (SnmpOidCopy(pSrcOid, pNewOid) != SM_SNMP_STATUS_NO_ERROR)
	{
		SMFreeMem(pNewOid);
		return NULL;
	}

	return pNewOid;
} // SnmpOidAllocAndCopy


//=============================================================================
// SnmpOidFree - free memory for SNMP OID
//=============================================================================
void SnmpOidFree(_INOUT SMSnmpOid * pOid,
				 _IN booln freeStruct)
{

	// free memory for IDs
	if (pOid->ids != NULL)
	{
		SMFreeMem(pOid->ids);
		pOid->ids = NULL;
		pOid->numIds = 0;
	}

	// free memory for SMSnmpOid struct
	if (freeStruct == TRUE)
	{
		SMFreeMem(pOid);
	}
} // SnmpOidFree


//=============================================================================
// SnmpOidCmp - compare two SNMP OIDs
//
// Returns: <0 if OID1 is less than OID2
//           0 if OID1 equals OID2
//          >0 if OID1 is greater than OID2
//=============================================================================
s32 SnmpOidCmp(_IN const SMSnmpOid * pOid1,
			   _IN const SMSnmpOid * pOid2)
{
	u32 numIdsCmp;
	s32 tempVal;


	// number of IDs to compare = smaller number of Ids of the two OIDs
	numIdsCmp = (pOid1->numIds < pOid2->numIds ? pOid1->numIds : pOid2->numIds);

	// compare the IDs
	tempVal = SnmpOidNCmp(pOid1, pOid2, numIdsCmp);
	if (tempVal != 0)
	{
		// difference found
		return tempVal;
	}

	// no differences found in IDs compared;
	// compare the number of IDs
	return pOid1->numIds - pOid2->numIds;
} // SnmpOidCmp


//=============================================================================
// SnmpOidNCmp - compare specified number of IDs of two SNMP OIDs
//
// Returns: <0 if OID1 is less than OID2
//           0 if OID1 equals OID2
//          >0 if OID1 is greater than OID2
//=============================================================================
s32 SnmpOidNCmp(_IN const SMSnmpOid * pOid1,
				_IN const SMSnmpOid * pOid2,
				_IN u32 numIdsCmp)
{
	u32 i;
	s32 tempVal;


	// compare the specified number of IDs
	for (i = 0; i < numIdsCmp; i++)
	{
		tempVal = (s32)(pOid1->ids[i] - pOid2->ids[i]);
		if (tempVal != 0)
		{
			// difference found
			return tempVal;
		}
	}

	// no differences found in IDs compared
	return 0;
} // SnmpOidNCmp


//=============================================================================
// SnmpOidAStrToIds - convert SNMP OID from dotted string to binary format
//=============================================================================
s32 SnmpOidAStrToIds(_OUT u32 * ids,
					 _INOUT u32 * pNumIds,
					 _IN const astring * pAStr)
{
	u32 maxIds;
	u32 numIds;
	u32 numDigits;
	unsigned int tempVal;
	astring currChar;
	astring lastChar;
	astring tempBuf[MAX_SNMP_OID_ID_DIGITS + 1];


	// sanity check
	if ((ids == NULL) || (*pNumIds == 0) || (pAStr == NULL))
	{
		SysDbgMsg3(("SnmpOidAStrToIds: ERROR: invalid parameter\n"));
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// initialize local variables
	maxIds = *pNumIds;
	numIds = 0;
	numDigits = 0;
	lastChar = '\0';

	// parse dotted string
	while ((currChar = *pAStr++) != '\0')
	{
		// validate current character
		if (currChar == '.')
		{
			if (lastChar == '.')
			{
				SysDbgMsg3(("SnmpOidAStrToIds: ERROR: two dots together\n"));
				return SM_SNMP_STATUS_BAD_VALUE;
			}
		}
		else if (isdigit(currChar) == 0)
		{
			SysDbgMsg3(("SnmpOidAStrToIds: ERROR: non-decimal character\n"));
			return SM_SNMP_STATUS_BAD_VALUE;
		}
		else
		{
			// current character is valid decimal digit
			if (numDigits == MAX_SNMP_OID_ID_DIGITS)
			{
				SysDbgMsg3(("SnmpOidAStrToIds: ERROR: ID value too big\n"));
				return SM_SNMP_STATUS_BAD_VALUE;
			}
			tempBuf[numDigits++] = currChar;
		}
		lastChar = currChar;

		// do we have a complete ID?
		if (((currChar == '.') || (*pAStr == '\0')) &&
			(numDigits != 0))
		{
			// yes, is there room for another ID?
			if (numIds == maxIds)
			{
				SysDbgMsg3(("SnmpOidAStrToIds: ERROR: too many IDs\n"));
				return SM_SNMP_STATUS_BAD_VALUE;
			}

			// null-terminate ID string
			tempBuf[numDigits] = '\0';

			// convert string to integer
			sscanf_s(tempBuf, "%u", &tempVal);

			// put integer in IDs buffer
			ids[numIds++] = (u32)tempVal;

			// reset
			numDigits = 0;
		}
	} // while ()

	// return number of IDs
	if (numIds == 0)
	{
		SysDbgMsg3(("SnmpOidAStrToIds: ERROR: no IDs\n"));
		return SM_SNMP_STATUS_BAD_VALUE;
	}
	*pNumIds = numIds;

	return SM_SNMP_STATUS_NO_ERROR;
} // SnmpOidAStrToIds


//=============================================================================
// SnmpOidToAStr - convert SNMP OID from binary to dotted string format
//=============================================================================
s32 SnmpOidToAStr(_IN const SMSnmpOid * pOid,
				  _OUT astring * pABuf,
				  _IN u32 aBufSize)
{

	// check parameters
	if ((pOid == NULL) || (pABuf == NULL))
	{
		SysDbgMsg3(("SnmpOidToAStr: ERROR: invalid parameter\n"));
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// convert OID
	return SnmpIdsToAStr(pOid->ids, pOid->numIds, pABuf, aBufSize);
} // SnmpIdsToAStr


//=============================================================================
// SnmpIdsToAStr - convert SNMP IDs from binary to dotted string format
//=============================================================================
s32 SnmpIdsToAStr(_IN u32 * ids,
				  _IN u32 numIds,
				  _OUT astring * pABuf,
				  _IN u32 aBufSize)
{
	u32 ii;
	u32 len;
	int charsWritten;


	// check parameters
	if ((ids == NULL) || (pABuf == NULL))
	{
		SysDbgMsg3(("SnmpIdsToAStr: ERROR: invalid parameter\n"));
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// prepare to convert IDs
	pABuf[0] = '\0';
	len = 0;

	// convert IDs to string format
	for (ii = 0; ii < numIds; ii++)
	{
		// convert and append the next ID
		charsWritten = sprintf_s(&pABuf[len],
			(aBufSize - len),
			"%u.",
			ids[ii]);

		// check if value converted successfully
		if (charsWritten == (-1))
		{
			SysDbgMsg3(("SnmpIdsToAStr: sprintf_s failed: ii: %u numIds: %u aBufSize: %u\n",
				ii,
				numIds,
				aBufSize));
			return SM_SNMP_STATUS_GEN_ERR;
		}

		// update value length
		len += (u32)(charsWritten);
	} // for (ii = 0; ii < numIds; ii++)

	// remove the last dot
	if (len != 0)
	{
		pABuf[len - 1] = '\0';
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // SnmpIdsToAStr


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

